@@ -34,7 +34,7 @@
 
 #include <stdio.h>
 
-WINE_DEFAULT_DEBUG_CHANNEL(d3dxof);
+WINE_DEFAULT_DEBUG_CHANNEL(d3dxof_parsing);
 
 #define TOKEN_NAME         1
 #define TOKEN_STRING       2
@@ -1020,7 +1020,6 @@ static BOOL parse_object_members_list(parse_buffer * buf)
   DWORD token;
   int i;
   xtemplate* pt = buf->pxt[buf->level];
-  DWORD last_dword = 0;
 
   for (i = 0; i < pt->nb_members; i++)
   {
@@ -1092,7 +1091,6 @@ static BOOL parse_object_members_list(parse_buffer * buf)
         if (token == TOKEN_INTEGER)
         {
           get_TOKEN(buf);
-          last_dword = *(DWORD*)buf->value;
           TRACE("%s = %d\n", pt->members[i].name, *(DWORD*)buf->value);
           /* Assume larger size */
           if (!check_buffer(buf, 4))
@@ -1163,7 +1161,7 @@ static BOOL parse_object_members_list(parse_buffer * buf)
       }
     }
 
-    if (buf->txt && (check_TOKEN(buf) != TOKEN_CBRACE))
+    if (nb_elems && buf->txt && (check_TOKEN(buf) != TOKEN_CBRACE))
     {
       token = get_TOKEN(buf);
       if ((token != TOKEN_SEMICOLON) && (token != TOKEN_COMMA))
@@ -1176,6 +1174,8 @@ static BOOL parse_object_members_list(parse_buffer * buf)
 
 static BOOL parse_object_parts(parse_buffer * buf, BOOL allow_optional)
 {
+  buf->pxo->nb_childs = 0;
+
   if (!parse_object_members_list(buf))
     return FALSE;
 
@@ -1198,7 +1198,7 @@ static BOOL parse_object_parts(parse_buffer * buf, BOOL allow_optional)
         if (get_TOKEN(buf) != TOKEN_CBRACE)
           return FALSE;
         TRACE("Found optional reference %s\n", (char*)buf->value);
-        for (i = 0; i < buf->nb_pxo_globals; i++)
+        for (i = 0; i < (buf->nb_pxo_globals+1); i++)
         {
           for (j = 0; j < (buf->pxo_globals[i])[0].nb_subobjects; j++)
           {
@@ -1207,19 +1207,19 @@ static BOOL parse_object_parts(parse_buffer * buf, BOOL allow_optional)
           }
         }
 _exit:
-        if (i == buf->nb_pxo_globals)
+        if (i == (buf->nb_pxo_globals+1))
         {
           ERR("Reference to unknown object %s\n", (char*)buf->value);
           return FALSE;
         }
-        buf->pxo->childs[buf->pxo->nb_childs] = &buf->pxo_tab[buf->cur_subobject++];
+        buf->pxo->childs[buf->pxo->nb_childs] = &buf->pxo_tab[buf->pxo->root->nb_subobjects++];
         buf->pxo->childs[buf->pxo->nb_childs]->ptarget = &(buf->pxo_globals[i])[j];
         buf->pxo->nb_childs++;
       }
       else if (check_TOKEN(buf) == TOKEN_NAME)
       {
         xobject* pxo = buf->pxo;
-        buf->pxo = buf->pxo->childs[buf->pxo->nb_childs] = &buf->pxo_tab[buf->cur_subobject++];
+        buf->pxo = buf->pxo->childs[buf->pxo->nb_childs] = &buf->pxo_tab[buf->pxo->root->nb_subobjects++];
 
         TRACE("Enter optional %s\n", (char*)buf->value);
         buf->level++;
