@@ -53,7 +53,7 @@ HRESULT WINAPI JoystickAGenericImpl_SetProperty(
     if (TRACE_ON(dinput))
         _dump_DIPROPHEADER(ph);
 
-    if (!HIWORD(rguid)) {
+    if (IS_DIPROP(rguid)) {
         switch (LOWORD(rguid)) {
         case (DWORD_PTR)DIPROP_RANGE: {
             LPCDIPROPRANGE pr = (LPCDIPROPRANGE)ph;
@@ -230,7 +230,7 @@ HRESULT WINAPI JoystickAGenericImpl_GetProperty(
     if (TRACE_ON(dinput))
         _dump_DIPROPHEADER(pdiph);
 
-    if (!HIWORD(rguid)) {
+    if (IS_DIPROP(rguid)) {
         switch (LOWORD(rguid)) {
         case (DWORD_PTR) DIPROP_RANGE: {
             LPDIPROPRANGE pr = (LPDIPROPRANGE)pdiph;
@@ -448,7 +448,7 @@ DWORD joystick_map_pov(POINTL *p)
  * Setup the dinput options.
  */
 
-HRESULT setup_dinput_options(JoystickGenericImpl *This, const BYTE *default_axis_map)
+HRESULT setup_dinput_options(JoystickGenericImpl *This, const int *default_axis_map)
 {
     char buffer[MAX_PATH+16];
     HKEY hkey, appkey;
@@ -549,7 +549,9 @@ HRESULT setup_dinput_options(JoystickGenericImpl *This, const BYTE *default_axis
             {
                 This->axis_map[i] = default_axis_map[i];
                 tokens = default_axis_map[i];
-                if (tokens >= 0 && tokens < 8)
+                if (tokens < 0)
+                    continue;
+                if (tokens < 8)
                     axis++;
                 else if (tokens < 15)
                 {
