@@ -44,6 +44,7 @@ static LRESULT Help_OnSize(HWND hWnd);
 #define TAB_TOP_PADDING     8
 #define TAB_RIGHT_PADDING   4
 #define TAB_MARGIN  8
+#define EDIT_HEIGHT         20
 
 static const WCHAR szEmpty[] = {0};
 
@@ -341,9 +342,6 @@ static void ResizeTabChild(HHInfo *info, int tab)
     SetWindowPos(hwnd, NULL, rect.left, rect.top, width, height,
                  SWP_NOZORDER | SWP_NOACTIVATE);
 
-    /* Resize the tab widget column to perfectly fit the tab window and
-     * leave sufficient space for the scroll widget.
-     */
     switch (tab)
     {
     case TAB_INDEX: {
@@ -351,11 +349,33 @@ static void ResizeTabChild(HHInfo *info, int tab)
         int border_width = GetSystemMetrics(SM_CXBORDER);
         int edge_width = GetSystemMetrics(SM_CXEDGE);
 
+        /* Resize the tab widget column to perfectly fit the tab window and
+         * leave sufficient space for the scroll widget.
+         */
         SendMessageW(info->tabs[TAB_INDEX].hwnd, LVM_SETCOLUMNWIDTH, 0,
                      width-scroll_width-2*border_width-2*edge_width);
 
         break;
     }
+    case TAB_SEARCH: {
+        int scroll_width = GetSystemMetrics(SM_CXVSCROLL);
+        int border_width = GetSystemMetrics(SM_CXBORDER);
+        int edge_width = GetSystemMetrics(SM_CXEDGE);
+        int top_pos = 0;
+
+        SetWindowPos(info->search.hwndEdit, NULL, 0, top_pos, width,
+                      EDIT_HEIGHT, SWP_NOZORDER | SWP_NOACTIVATE);
+        top_pos += EDIT_HEIGHT + TAB_MARGIN;
+        SetWindowPos(info->search.hwndList, NULL, 0, top_pos, width,
+                      height-top_pos, SWP_NOZORDER | SWP_NOACTIVATE);
+        /* Resize the tab widget column to perfectly fit the tab window and
+         * leave sufficient space for the scroll widget.
+         */
+        SendMessageW(info->search.hwndList, LVM_SETCOLUMNWIDTH, 0,
+                     width-scroll_width-2*border_width-2*edge_width);
+
+        break;
+    }
     }
 }
 
@@ -401,6 +421,7 @@ static LRESULT OnTopicChange(HHInfo *info, void *user_data)
 {
     LPCWSTR chmfile = NULL, name = NULL, local = NULL;
     ContentItem *citer;
+    SearchItem *siter;
     IndexItem *iiter;
 
     if(!user_data || !info)
@@ -452,6 +473,12 @@ static LRESULT OnTopicChange(HHInfo *info, void *user_data)
         local = iiter->items[0].local;
         chmfile = iiter->merge.chm_file;
         break;
+    case TAB_SEARCH:
+        siter = (SearchItem *) user_data;
+        name = siter->filename;
+        local = siter->filename;
+        chmfile = info->pCHMInfo->szFile;
+        break;
     default:
         FIXME("Unhandled operation for this tab!\n");
         return 0;
@@ -469,6 +496,22 @@ static LRESULT OnTopicChange(HHInfo *info, void *user_data)
     return 0;
 }
 
+/* Capture the Enter/Return key and send it up to Child_WndProc as an NM_RETURN message */
+static LRESULT CALLBACK EditChild_WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
+{
+    WNDPROC editWndProc = (WNDPROC)GetWindowLongPtrW(hWnd, GWLP_USERDATA);
+
+    if(message == WM_KEYUP && wParam == VK_RETURN)
+    {
+        NMHDR nmhdr;
+
+        nmhdr.hwndFrom = hWnd;
+        nmhdr.code = NM_RETURN;
+        SendMessageW(GetParent(GetParent(hWnd)), WM_NOTIFY, wParam, (LPARAM)&nmhdr);
+    }
+    return editWndProc(hWnd, message, wParam, lParam);
+}
+
 static LRESULT CALLBACK Child_WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 {
     switch (message)
@@ -487,8 +530,62 @@ static LRESULT CALLBACK Child_WndProc(HWND hWnd, UINT message, WPARAM wParam, LP
         case TVN_SELCHANGEDW:
             return OnTopicChange(info, (void*)((NMTREEVIEWW *)lParam)->itemNew.lParam);
         case NM_DBLCLK:
-            if(info->current_tab == TAB_INDEX)
+            if(!info)
+                return 0;
+            switch(info->current_tab)
+            {
+            case TAB_INDEX:
                 return OnTopicChange(info, (void*)((NMITEMACTIVATE *)lParam)->lParam);
+            case TAB_SEARCH:
+                return OnTopicChange(info, (void*)((NMITEMACTIVATE *)lParam)->lParam);
+            }
+            break;
+        case NM_RETURN:
+            if(!info)
+                return 0;
+            switch(info->current_tab) {
+            case TAB_INDEX: {
+                HWND hwndList = info->tabs[TAB_INDEX].hwnd;
+                LVITEMW lvItem;
+
+                lvItem.iItem = (int) SendMessageW(hwndList, LVM_GETSELECTIONMARK, 0, 0);
+                lvItem.mask = TVIF_PARAM;
+                SendMessageW(hwndList, LVM_GETITEMW, 0, (LPARAM)&lvItem);
+                OnTopicChange(info, (void*) lvItem.lParam);
+                return 0;
+            }
+            case TAB_SEARCH: {
+                if(nmhdr->hwndFrom == info->search.hwndEdit) {
+                    char needle[100];
+                    DWORD i, len;
+
+                    len = GetWindowTextA(info->search.hwndEdit, needle, sizeof(needle));
+                    if(!len)
+                    {
+                        FIXME("Unable to get search text.\n");
+                        return 0;
+                    }
+                    /* Convert the requested text for comparison later against the
+                     * lower case version of HTML file contents.
+                     */
+                    for(i=0;i<len;i++)
+                        needle[i] = tolower(needle[i]);
+                    InitSearch(info, needle);
+                    return 0;
+                }else if(nmhdr->hwndFrom == info->search.hwndList) {
+                    HWND hwndList = info->search.hwndList;
+                    LVITEMW lvItem;
+
+                    lvItem.iItem = (int) SendMessageW(hwndList, LVM_GETSELECTIONMARK, 0, 0);
+                    lvItem.mask = TVIF_PARAM;
+                    SendMessageW(hwndList, LVM_GETITEMW, 0, (LPARAM)&lvItem);
+                    OnTopicChange(info, (void*) lvItem.lParam);
+                    return 0;
+                }
+                break;
+            }
+            }
+            break;
         }
         break;
     }
@@ -850,6 +947,69 @@ static BOOL AddIndexTab(HHInfo *info)
     return TRUE;
 }
 
+static BOOL AddSearchTab(HHInfo *info)
+{
+    HWND hwndList, hwndEdit, hwndContainer;
+    char hidden_column[] = "Column";
+    WNDPROC editWndProc;
+    LVCOLUMNA lvc;
+
+    if(info->tabs[TAB_SEARCH].id == -1)
+        return TRUE; /* No "Search" tab */
+    hwndContainer = CreateWindowExW(WS_EX_CONTROLPARENT, szChildClass, szEmpty,
+                                    WS_CHILD, 0, 0, 0, 0, info->WinType.hwndNavigation,
+                                    NULL, hhctrl_hinstance, NULL);
+    if(!hwndContainer) {
+        ERR("Could not create search window container control.\n");
+        return FALSE;
+    }
+    hwndEdit = CreateWindowExW(WS_EX_CLIENTEDGE, WC_EDITW, szEmpty, WS_CHILD
+                                | WS_VISIBLE | ES_LEFT | SS_NOTIFY, 0, 0, 0, 0,
+                               hwndContainer, NULL, hhctrl_hinstance, NULL);
+    if(!hwndEdit) {
+        ERR("Could not create search ListView control.\n");
+        return FALSE;
+    }
+    if(SendMessageW(hwndEdit, WM_SETFONT, (WPARAM) info->hFont, (LPARAM) FALSE) == -1)
+    {
+        ERR("Could not set font for edit control.\n");
+        return FALSE;
+    }
+    editWndProc = (WNDPROC) SetWindowLongPtrW(hwndEdit, GWLP_WNDPROC, (LONG_PTR)EditChild_WndProc);
+    if(!editWndProc) {
+        ERR("Could not redirect messages for edit control.\n");
+        return FALSE;
+    }
+    SetWindowLongPtrW(hwndEdit, GWLP_USERDATA, (LONG_PTR)editWndProc);
+    hwndList = CreateWindowExW(WS_EX_CLIENTEDGE, WC_LISTVIEWW, szEmpty,
+                               WS_CHILD | WS_VISIBLE | WS_BORDER | LVS_SINGLESEL
+                                | LVS_REPORT | LVS_NOCOLUMNHEADER, 0, 0, 0, 0,
+                               hwndContainer, NULL, hhctrl_hinstance, NULL);
+    if(!hwndList) {
+        ERR("Could not create search ListView control.\n");
+        return FALSE;
+    }
+    memset(&lvc, 0, sizeof(lvc));
+    lvc.mask = LVCF_TEXT;
+    lvc.pszText = hidden_column;
+    if(SendMessageW(hwndList, LVM_INSERTCOLUMNA, 0, (LPARAM) &lvc) == -1)
+    {
+        ERR("Could not create ListView column\n");
+        return FALSE;
+    }
+
+    info->search.hwndEdit = hwndEdit;
+    info->search.hwndList = hwndList;
+    info->search.hwndContainer = hwndContainer;
+    info->tabs[TAB_SEARCH].hwnd = hwndContainer;
+
+    SetWindowLongPtrW(hwndContainer, GWLP_USERDATA, (LONG_PTR)info);
+
+    ResizeTabChild(info, TAB_SEARCH);
+
+    return TRUE;
+}
+
 /* The Index tab's sub-topic popup */
 
 static void ResizePopupChild(HHInfo *info)
@@ -910,7 +1070,9 @@ static LRESULT CALLBACK PopupChild_WndProc(HWND hWnd, UINT message, WPARAM wPara
     {
     case WM_NOTIFY: {
         NMHDR *nmhdr = (NMHDR*)lParam;
-        if(nmhdr->code == NM_DBLCLK) {
+        switch(nmhdr->code)
+        {
+        case NM_DBLCLK: {
             HHInfo *info = (HHInfo*)GetWindowLongPtrW(hWnd, GWLP_USERDATA);
             IndexSubItem *iter;
 
@@ -923,6 +1085,23 @@ static LRESULT CALLBACK PopupChild_WndProc(HWND hWnd, UINT message, WPARAM wPara
             ShowWindow(info->popup.hwndPopup, SW_HIDE);
             return 0;
         }
+        case NM_RETURN: {
+            HHInfo *info = (HHInfo*)GetWindowLongPtrW(hWnd, GWLP_USERDATA);
+            IndexSubItem *iter;
+            LVITEMW lvItem;
+
+            if(info == 0)
+                return 0;
+
+            lvItem.iItem = (int) SendMessageW(info->popup.hwndList, LVM_GETSELECTIONMARK, 0, 0);
+            lvItem.mask = TVIF_PARAM;
+            SendMessageW(info->popup.hwndList, LVM_GETITEMW, 0, (LPARAM)&lvItem);
+            iter = (IndexSubItem*) lvItem.lParam;
+            NavigateToChm(info, info->index->merge.chm_file, iter->local);
+            ShowWindow(info->popup.hwndPopup, SW_HIDE);
+            return 0;
+        }
+        }
         break;
     }
     default:
@@ -1201,6 +1380,9 @@ static BOOL CreateViewer(HHInfo *pHHInfo)
     if (!AddIndexPopup(pHHInfo))
         return FALSE;
 
+    if (!AddSearchTab(pHHInfo))
+        return FALSE;
+
     InitContent(pHHInfo);
     InitIndex(pHHInfo);
 
@@ -1232,6 +1414,7 @@ void ReleaseHelpViewer(HHInfo *info)
     ReleaseWebBrowser(info);
     ReleaseContent(info);
     ReleaseIndex(info);
+    ReleaseSearch(info);
 
     if(info->WinType.hwndHelp)
         DestroyWindow(info->WinType.hwndHelp);
