@@ -35,7 +35,7 @@ static void testInit(void)
     pInit = (void *)GetProcAddress(inetmib1, "SnmpExtensionInit");
     if (!pInit)
     {
-        skip("no SnmpExtensionInit\n");
+        win_skip("no SnmpExtensionInit\n");
         return;
     }
     /* Crash
@@ -47,6 +47,8 @@ static void testInit(void)
     ok(ret, "SnmpExtensionInit failed: %d\n", GetLastError());
     ok(!strcmp("1.3.6.1.2.1.1", SnmpUtilOidToA(&oid)),
         "Expected 1.3.6.1.2.1.1, got %s\n", SnmpUtilOidToA(&oid));
+
+    SnmpUtilOidFree(&oid);
 }
 
 static void testQuery(void)
@@ -72,7 +74,7 @@ static void testQuery(void)
     pQuery = (void *)GetProcAddress(inetmib1, "SnmpExtensionQuery");
     if (!pQuery)
     {
-        skip("couldn't find SnmpExtensionQuery\n");
+        win_skip("couldn't find SnmpExtensionQuery\n");
         return;
     }
     /* Crash
@@ -88,7 +90,7 @@ static void testQuery(void)
     error = 0xdeadbeef;
     index = 0xdeadbeef;
     ret = pQuery(SNMP_PDU_GET, &list, &error, &index);
-    ok(ret, "SnmpExtensionQuery failed: %d\n", GetLastError());
+    ok(ret, "SnmpExtensionQuery failed: %d, %d\n", error, index);
     ok(error == SNMP_ERRORSTATUS_NOERROR,
         "expected SNMP_ERRORSTATUS_NOERROR, got %d\n", error);
     ok(index == 0, "expected index 0, got %d\n", index);
@@ -105,7 +107,7 @@ static void testQuery(void)
     error = 0xdeadbeef;
     index = 0xdeadbeef;
     ret = pQuery(SNMP_PDU_GET, &list, &error, &index);
-    ok(ret, "SnmpExtensionQuery failed: %d\n", GetLastError());
+    ok(ret, "SnmpExtensionQuery failed: %d, %d\n", error, index);
     ok(error == SNMP_ERRORSTATUS_NOERROR ||
         error == ERROR_FILE_NOT_FOUND /* Win9x */,
         "expected SNMP_ERRORSTATUS_NOERROR or ERROR_FILE_NOT_FOUND, got %d\n",
@@ -125,7 +127,7 @@ static void testQuery(void)
     error = 0xdeadbeef;
     index = 0xdeadbeef;
     ret = pQuery(SNMP_PDU_GET, &list, &error, &index);
-    ok(ret, "SnmpExtensionQuery failed: %d\n", GetLastError());
+    ok(ret, "SnmpExtensionQuery failed: %d, %d\n", error, index);
     ok(error == SNMP_ERRORSTATUS_NOSUCHNAME,
         "expected SNMP_ERRORSTATUS_NOSUCHNAME, got %d\n", error);
     /* The index is 1-based rather than 0-based */
@@ -147,7 +149,7 @@ static void testQuery(void)
     error = 0xdeadbeef;
     index = 0xdeadbeef;
     ret = pQuery(SNMP_PDU_GET, &list, &error, &index);
-    ok(ret, "SnmpExtensionQuery failed: %d\n", GetLastError());
+    ok(ret, "SnmpExtensionQuery failed: %d, %d\n", error, index);
     ok(error == SNMP_ERRORSTATUS_NOSUCHNAME,
         "expected SNMP_ERRORSTATUS_NOSUCHNAME, got %d\n", error);
     ok(index == 1, "expected index 1, got %d\n", index);
@@ -169,7 +171,7 @@ static void testQuery(void)
         error = 0xdeadbeef;
         index = 0xdeadbeef;
         ret = pQuery(SNMP_PDU_GETNEXT, &list, &error, &index);
-        ok(ret, "SnmpExtensionQuery failed: %d\n", GetLastError());
+        ok(ret, "SnmpExtensionQuery failed: %d, %d\n", error, index);
         ok(error == SNMP_ERRORSTATUS_NOERROR,
             "expected SNMP_ERRORSTATUS_NOERROR, got %d\n", error);
         ok(index == 0, "expected index 0, got %d\n", index);
@@ -252,7 +254,7 @@ static void testQuery(void)
     moreData = TRUE;
     noChange = FALSE;
     ret = pQuery(SNMP_PDU_GETNEXT, &list, &error, &index);
-    ok(ret, "SnmpExtensionQuery failed: %d\n", GetLastError());
+    ok(ret, "SnmpExtensionQuery failed: %d, %d\n", error, index);
     ok(error == SNMP_ERRORSTATUS_NOERROR,
         "expected SNMP_ERRORSTATUS_NOERROR, got %d\n", error);
     ok(index == 0, "expected index 0, got %d\n", index);
@@ -274,7 +276,7 @@ static void testQuery(void)
     moreData = TRUE;
     do {
         ret = pQuery(SNMP_PDU_GETNEXT, &list, &error, &index);
-        ok(ret, "SnmpExtensionQuery failed: %d\n", GetLastError());
+        ok(ret, "SnmpExtensionQuery failed: %d, %d\n", error, index);
         ok(error == SNMP_ERRORSTATUS_NOERROR,
             "expected SNMP_ERRORSTATUS_NOERROR, got %d\n", error);
         ok(index == 0, "expected index 0, got %d\n", index);
@@ -340,7 +342,7 @@ static void testQuery(void)
     noChange = FALSE;
     do {
         ret = pQuery(SNMP_PDU_GETNEXT, &list, &error, &index);
-        ok(ret, "SnmpExtensionQuery failed: %d\n", GetLastError());
+        ok(ret, "SnmpExtensionQuery failed: %d, %d\n", error, index);
         ok(error == SNMP_ERRORSTATUS_NOERROR,
             "expected SNMP_ERRORSTATUS_NOERROR, got %d\n", error);
         ok(index == 0, "expected index 0, got %d\n", index);
@@ -406,7 +408,7 @@ static void testQuery(void)
     noChange = FALSE;
     do {
         ret = pQuery(SNMP_PDU_GETNEXT, &list, &error, &index);
-        ok(ret, "SnmpExtensionQuery failed: %d\n", GetLastError());
+        ok(ret, "SnmpExtensionQuery failed: %d, %d\n", error, index);
         /* FIXME:  error and index aren't checked here because the UDP table is
          * the last OID currently supported by Wine, so the last GetNext fails.
          * todo_wine is also not effective because it will succeed for all but
@@ -473,4 +475,5 @@ START_TEST(main)
     inetmib1 = LoadLibraryA("inetmib1");
     testInit();
     testQuery();
+    FreeLibrary(inetmib1);
 }
