@@ -42,6 +42,8 @@
 # include <resolv.h>
 #endif
 
+#define NONAMELESSUNION
+#define NONAMELESSSTRUCT
 #include "windef.h"
 #include "winbase.h"
 #include "winreg.h"
@@ -64,16 +66,13 @@ WINE_DEFAULT_DEBUG_CHANNEL(iphlpapi);
 #define INADDR_NONE ~0UL
 #endif
 
-static int resolver_initialised;
-
 /* call res_init() just once because of a bug in Mac OS X 10.4 */
+/* Call once per thread on systems that have per-thread _res. */
+/* FIXME: should do same fix in dnsapi (or use dnsapi here?) */
 static void initialise_resolver(void)
 {
-    if (!resolver_initialised)
-    {
+    if ((_res.options & RES_INIT) == 0)
         res_init();
-        resolver_initialised = 1;
-    }
 }
 
 BOOL WINAPI DllMain (HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
@@ -642,22 +641,29 @@ static ULONG adapterAddressesFromIndex(DWORD index, IP_ADAPTER_ADDRESSES *aa, UL
 
     total_size = sizeof(IP_ADAPTER_ADDRESSES);
     total_size += IF_NAMESIZE;
+    total_size += IF_NAMESIZE * sizeof(WCHAR);
     total_size += sizeof(IP_ADAPTER_UNICAST_ADDRESS) * num_addrs;
     total_size += sizeof(struct sockaddr_in) * num_addrs;
 
     if (aa && *size >= total_size)
     {
-        char name[IF_NAMESIZE], *ptr = (char *)aa + sizeof(IP_ADAPTER_ADDRESSES);
+        char name[IF_NAMESIZE], *ptr = (char *)aa + sizeof(IP_ADAPTER_ADDRESSES), *src;
+        WCHAR *dst;
         DWORD buflen, type, status;
 
         memset(aa, 0, sizeof(IP_ADAPTER_ADDRESSES));
-        aa->Length  = sizeof(IP_ADAPTER_ADDRESSES);
-        aa->IfIndex = index;
+        aa->u.s.Length  = sizeof(IP_ADAPTER_ADDRESSES);
+        aa->u.s.IfIndex = index;
 
         getInterfaceNameByIndex(index, name);
         memcpy(ptr, name, IF_NAMESIZE);
         aa->AdapterName = ptr;
         ptr += IF_NAMESIZE;
+        aa->FriendlyName = (WCHAR *)ptr;
+        for (src = name, dst = (WCHAR *)ptr; *src; src++, dst++)
+            *dst = *src;
+        *dst++ = 0;
+        ptr = (char *)dst;
 
         if (num_addrs)
         {
@@ -668,16 +674,16 @@ static ULONG adapterAddressesFromIndex(DWORD index, IP_ADAPTER_ADDRESSES *aa, UL
             for (i = 0; i < num_addrs; i++)
             {
                 memset(ua, 0, sizeof(IP_ADAPTER_UNICAST_ADDRESS));
-                ua->Length                  = sizeof(IP_ADAPTER_UNICAST_ADDRESS);
+                ua->u.s.Length              = sizeof(IP_ADAPTER_UNICAST_ADDRESS);
                 ua->Address.iSockaddrLength = sizeof(struct sockaddr_in);
-                ua->Address.lpSockaddr      = (SOCKADDR *)((char *)ua + ua->Length);
+                ua->Address.lpSockaddr      = (SOCKADDR *)((char *)ua + ua->u.s.Length);
 
                 sa = (struct sockaddr_in *)ua->Address.lpSockaddr;
                 sa->sin_family      = AF_INET;
                 sa->sin_addr.s_addr = addrs[i];
                 sa->sin_port        = 0;
 
-                ptr += ua->Length + ua->Address.iSockaddrLength;
+                ptr += ua->u.s.Length + ua->Address.iSockaddrLength;
                 if (i < num_addrs - 1)
                 {
                     ua->Next = (IP_ADAPTER_UNICAST_ADDRESS *)ptr;
@@ -860,9 +866,7 @@ DWORD WINAPI GetBestRoute(DWORD dwDestAddr, DWORD dwSourceAddr, PMIB_IPFORWARDRO
           matchedBits = numShifts;
           matchedNdx = ndx;
         }
-        else if (!matchedBits && table->table[ndx].dwForwardType ==
-         MIB_IPROUTE_TYPE_INDIRECT) {
-          /* default to a default gateway */
+        else if (!matchedBits) {
           matchedNdx = ndx;
         }
       }
