@@ -35,7 +35,6 @@
 #include "winbase.h"
 #include "winerror.h"
 #include "winternl.h"
-#include "wine/winbase16.h"
 #include "wine/exception.h"
 #include "wine/library.h"
 #include "wine/server.h"
@@ -652,3 +651,83 @@ DWORD WINAPI GetCurrentThreadId(void)
 }
 
 #endif  /* __i386__ */
+
+/*************************************************************************
+ *              rtlmode_to_win32mode
+ */
+static DWORD rtlmode_to_win32mode( DWORD rtlmode )
+{
+    DWORD win32mode = 0;
+
+    if (rtlmode & 0x10)
+        win32mode |= SEM_FAILCRITICALERRORS;
+    if (rtlmode & 0x20)
+        win32mode |= SEM_NOGPFAULTERRORBOX;
+    if (rtlmode & 0x40)
+        win32mode |= SEM_NOOPENFILEERRORBOX;
+
+    return win32mode;
+}
+
+/***********************************************************************
+ *              SetThreadErrorMode (KERNEL32.@)
+ *
+ * Set the thread local error mode.
+ *
+ * PARAMS
+ *  mode    [I] The new error mode, a bitwise or of SEM_FAILCRITICALERRORS,
+ *              SEM_NOGPFAULTERRORBOX and SEM_NOOPENFILEERRORBOX.
+ *  oldmode [O] Destination of the old error mode (may be NULL)
+ *
+ * RETURNS
+ *  Success: TRUE
+ *  Failure: FALSE, check GetLastError
+ */
+BOOL WINAPI SetThreadErrorMode( DWORD mode, LPDWORD oldmode )
+{
+    NTSTATUS status;
+    DWORD tmp = 0;
+
+    if (mode & ~(SEM_FAILCRITICALERRORS |
+                 SEM_NOGPFAULTERRORBOX |
+                 SEM_NOOPENFILEERRORBOX))
+    {
+        SetLastError( ERROR_INVALID_PARAMETER );
+        return FALSE;
+    }
+
+    if (mode & SEM_FAILCRITICALERRORS)
+        tmp |= 0x10;
+    if (mode & SEM_NOGPFAULTERRORBOX)
+        tmp |= 0x20;
+    if (mode & SEM_NOOPENFILEERRORBOX)
+        tmp |= 0x40;
+
+    status = RtlSetThreadErrorMode( tmp, oldmode );
+    if (status)
+    {
+        SetLastError( RtlNtStatusToDosError(status) );
+        return FALSE;
+    }
+
+    if (oldmode)
+        *oldmode = rtlmode_to_win32mode(*oldmode);
+
+    return TRUE;
+}
+
+/***********************************************************************
+ *              GetThreadErrorMode (KERNEL32.@)
+ *
+ * Get the thread local error mode.
+ *
+ * PARAMS
+ *  None.
+ *
+ * RETURNS
+ *  The current thread local error mode.
+ */
+DWORD WINAPI GetThreadErrorMode( void )
+{
+    return rtlmode_to_win32mode( RtlGetThreadErrorMode() );
+}
