@@ -460,29 +460,69 @@ static HRESULT WINAPI HTMLImgElement_get_name(IHTMLImgElement *iface, BSTR *p)
 static HRESULT WINAPI HTMLImgElement_put_width(IHTMLImgElement *iface, LONG v)
 {
     HTMLImgElement *This = HTMLIMG_THIS(iface);
-    FIXME("(%p)->(%d)\n", This, v);
-    return E_NOTIMPL;
+    nsresult nsres;
+
+    TRACE("(%p)->(%d)\n", This, v);
+
+    nsres = nsIDOMHTMLImageElement_SetWidth(This->nsimg, v);
+    if(NS_FAILED(nsres)) {
+        ERR("SetWidth failed: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_width(IHTMLImgElement *iface, LONG *p)
 {
     HTMLImgElement *This = HTMLIMG_THIS(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+    PRInt32 width;
+    nsresult nsres;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    nsres = nsIDOMHTMLImageElement_GetWidth(This->nsimg, &width);
+    if(NS_FAILED(nsres)) {
+        ERR("GetWidth failed: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    *p = width;
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLImgElement_put_height(IHTMLImgElement *iface, LONG v)
 {
     HTMLImgElement *This = HTMLIMG_THIS(iface);
-    FIXME("(%p)->(%d)\n", This, v);
-    return E_NOTIMPL;
+    nsresult nsres;
+
+    TRACE("(%p)->(%d)\n", This, v);
+
+    nsres = nsIDOMHTMLImageElement_SetHeight(This->nsimg, v);
+    if(NS_FAILED(nsres)) {
+        ERR("SetHeight failed: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_height(IHTMLImgElement *iface, LONG *p)
 {
     HTMLImgElement *This = HTMLIMG_THIS(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+    PRInt32 height;
+    nsresult nsres;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    nsres = nsIDOMHTMLImageElement_GetHeight(This->nsimg, &height);
+    if(NS_FAILED(nsres)) {
+        ERR("GetHeight failed: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    *p = height;
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLImgElement_put_start(IHTMLImgElement *iface, BSTR v)
@@ -724,12 +764,38 @@ static HRESULT WINAPI HTMLImageElementFactory_Invoke(IHTMLImageElementFactory *i
     return E_NOTIMPL;
 }
 
+static LONG var_to_size(const VARIANT *v)
+{
+    switch(V_VT(v)) {
+    case VT_EMPTY:
+        return 0;
+    case VT_I4:
+        return V_I4(v);
+    case VT_BSTR: {
+        LONG ret;
+        HRESULT hres;
+
+        hres = VarI4FromStr(V_BSTR(v), 0, 0, &ret);
+        if(FAILED(hres)) {
+            FIXME("VarI4FromStr failed: %08x\n", hres);
+            return 0;
+        }
+        return ret;
+    }
+    default:
+        FIXME("unsupported size %s\n", debugstr_variant(v));
+    }
+    return 0;
+}
+
 static HRESULT WINAPI HTMLImageElementFactory_create(IHTMLImageElementFactory *iface,
         VARIANT width, VARIANT height, IHTMLImgElement **img_elem)
 {
     HTMLImageElementFactory *This = HTMLIMGFACTORY_THIS(iface);
+    IHTMLImgElement *img;
     HTMLElement *elem;
     nsIDOMHTMLElement *nselem;
+    LONG l;
     HRESULT hres;
 
     static const PRUnichar imgW[] = {'I','M','G',0};
@@ -754,7 +820,7 @@ static HRESULT WINAPI HTMLImageElementFactory_create(IHTMLImageElementFactory *i
         return E_FAIL;
     }
 
-    hres = IHTMLElement_QueryInterface(HTMLELEM(elem), &IID_IHTMLImgElement, (void**)img_elem);
+    hres = IHTMLElement_QueryInterface(HTMLELEM(elem), &IID_IHTMLImgElement, (void**)&img);
     if(FAILED(hres)) {
         ERR("IHTMLElement_QueryInterface failed: 0x%08x\n", hres);
         return hres;
@@ -762,9 +828,14 @@ static HRESULT WINAPI HTMLImageElementFactory_create(IHTMLImageElementFactory *i
 
     nsIDOMHTMLElement_Release(nselem);
 
-    if(V_VT(&width) != VT_EMPTY || V_VT(&height) != VT_EMPTY)
-        FIXME("Not setting image dimensions\n");
+    l = var_to_size(&width);
+    if(l)
+        IHTMLImgElement_put_width(img, l);
+    l = var_to_size(&height);
+    if(l)
+        IHTMLImgElement_put_height(img, l);
 
+    *img_elem = img;
     return S_OK;
 }
 
