@@ -322,6 +322,8 @@ static void parse_complete_proc(task_t *task)
         init_editor(&doc->basedoc);
 
     call_explorer_69(doc);
+    if(doc->view_sink)
+        IAdviseSink_OnViewChange(doc->view_sink, DVASPECT_CONTENT, -1);
     call_property_onchanged(&doc->basedoc.cp_propnotif, 1005);
     call_explorer_69(doc);
 
