@@ -42,17 +42,6 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(msi);
 
-
-extern const WCHAR szRegisterClassInfo[];
-extern const WCHAR szRegisterProgIdInfo[];
-extern const WCHAR szRegisterExtensionInfo[];
-extern const WCHAR szRegisterMIMEInfo[];
-
-extern const WCHAR szUnregisterClassInfo[];
-extern const WCHAR szUnregisterExtensionInfo[];
-extern const WCHAR szUnregisterMIMEInfo[];
-extern const WCHAR szUnregisterProgIdInfo[];
-
 static MSIAPPID *load_appid( MSIPACKAGE* package, MSIRECORD *row )
 {
     LPCWSTR buffer;
@@ -486,7 +475,7 @@ static MSIEXTENSION *load_given_extension( MSIPACKAGE *package, LPCWSTR name )
 
 static UINT iterate_load_verb(MSIRECORD *row, LPVOID param)
 {
-    MSIPACKAGE* package = (MSIPACKAGE*)param;
+    MSIPACKAGE* package = param;
     MSIVERB *verb;
     LPCWSTR buffer;
     MSIEXTENSION *extension;
@@ -527,7 +516,7 @@ static UINT iterate_all_classes(MSIRECORD *rec, LPVOID param)
     LPCWSTR clsid;
     LPCWSTR context;
     LPCWSTR buffer;
-    MSIPACKAGE* package =(MSIPACKAGE*)param;
+    MSIPACKAGE* package = param;
     MSICLASS *cls;
     BOOL match = FALSE;
 
@@ -577,7 +566,7 @@ static UINT iterate_all_extensions(MSIRECORD *rec, LPVOID param)
     MSICOMPONENT *comp;
     LPCWSTR buffer;
     LPCWSTR extension;
-    MSIPACKAGE* package =(MSIPACKAGE*)param;
+    MSIPACKAGE* package = param;
     BOOL match = FALSE;
     MSIEXTENSION *ext;
 
@@ -622,7 +611,7 @@ static VOID load_all_extensions(MSIPACKAGE *package)
 static UINT iterate_all_progids(MSIRECORD *rec, LPVOID param)
 {
     LPCWSTR buffer;
-    MSIPACKAGE* package =(MSIPACKAGE*)param;
+    MSIPACKAGE* package = param;
 
     buffer = MSI_RecordGetString(rec,1);
     load_given_progid(package,buffer);
@@ -666,7 +655,7 @@ static VOID load_all_verbs(MSIPACKAGE *package)
 static UINT iterate_all_mimes(MSIRECORD *rec, LPVOID param)
 {
     LPCWSTR buffer;
-    MSIPACKAGE* package =(MSIPACKAGE*)param;
+    MSIPACKAGE* package = param;
 
     buffer = MSI_RecordGetString(rec,1);
     load_given_mime(package,buffer);
@@ -798,7 +787,6 @@ UINT ACTION_RegisterClassInfo(MSIPACKAGE *package)
     static const WCHAR szProgID[] = { 'P','r','o','g','I','D',0 };
     static const WCHAR szVIProgID[] = { 'V','e','r','s','i','o','n','I','n','d','e','p','e','n','d','e','n','t','P','r','o','g','I','D',0 };
     static const WCHAR szAppID[] = { 'A','p','p','I','D',0 };
-    static const WCHAR szSpace[] = {' ',0};
     static const WCHAR szFileType_fmt[] = {'F','i','l','e','T','y','p','e','\\','%','s','\\','%','i',0};
     HKEY hkey,hkey2,hkey3;
     MSICLASS *cls;
@@ -821,16 +809,17 @@ UINT ACTION_RegisterClassInfo(MSIPACKAGE *package)
             continue;
 
         feature = cls->Feature;
+        if (!feature)
+            continue;
 
         /*
          * MSDN says that these are based on Feature not on Component.
          */
-        if (!ACTION_VerifyFeatureForAction( feature, INSTALLSTATE_LOCAL ) &&
-            !ACTION_VerifyFeatureForAction( feature, INSTALLSTATE_ADVERTISED ))
+        if (feature->ActionRequest != INSTALLSTATE_LOCAL &&
+            feature->ActionRequest != INSTALLSTATE_ADVERTISED )
         {
-            TRACE("Skipping class %s reg due to disabled feature %s\n",
-                  debugstr_w(cls->clsid), debugstr_w(feature->Feature));
-
+            TRACE("Feature %s not scheduled for installation, skipping regstration of class %s\n",
+                  debugstr_w(feature->Feature), debugstr_w(cls->clsid));
             continue;
         }
 
@@ -1098,7 +1087,6 @@ static UINT register_verb(MSIPACKAGE *package, LPCWSTR progid,
      strcpyW(command,advertise);
      if (verb->Argument)
      {
-         static const WCHAR szSpace[] = {' ',0};
          strcatW(command,szSpace);
          strcatW(command,verb->Argument);
      }
@@ -1155,18 +1143,18 @@ UINT ACTION_RegisterExtensionInfo(MSIPACKAGE *package)
             continue;
 
         feature = ext->Feature;
+        if (!feature)
+            continue;
 
         /* 
          * yes. MSDN says that these are based on _Feature_ not on
          * Component.  So verify the feature is to be installed
          */
-        if ((!ACTION_VerifyFeatureForAction( feature, INSTALLSTATE_LOCAL )) &&
-             !(install_on_demand &&
-               ACTION_VerifyFeatureForAction( feature, INSTALLSTATE_ADVERTISED )))
+        if (feature->ActionRequest != INSTALLSTATE_LOCAL &&
+            !(install_on_demand && feature->ActionRequest == INSTALLSTATE_ADVERTISED))
         {
-            TRACE("Skipping extension %s reg due to disabled feature %s\n",
-                   debugstr_w(ext->Extension), debugstr_w(feature->Feature));
-
+            TRACE("Feature %s not scheduled for installation, skipping registration of extension %s\n",
+                   debugstr_w(feature->Feature), debugstr_w(ext->Extension));
             continue;
         }
 
