@@ -48,10 +48,29 @@ static struct cp_extra_info_t g_cpextrainfo[] =
     {936, {0x40, 0xfe, 0, 0}},
     {949, {0x41, 0xfe, 0, 0}},
     {950, {0x40, 0x7e, 0xa1, 0xfe, 0, 0}},
+    {1361, {0x31, 0x7e, 0x81, 0xfe, 0, 0}},
     {20932, {1, 255, 0, 0}},  /* seems to give different results on different systems */
     {0, {1, 255, 0, 0}}       /* match all with FIXME */
 };
 
+/* Maps multibyte cp932 punctuation marks to single byte equivalents */
+static const unsigned char mbctombb_932_punct[] = {
+  0x20,0xa4,0xa1,0x2c,0x2e,0xa5,0x3a,0x3b,0x3f,0x21,0xde,0xdf,0x00,0x00,0x00,0x5e,
+  0x7e,0x5f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xb0,0x00,0x00,0x2f,0x00,
+  0x00,0x00,0x7c,0x00,0x00,0x60,0x27,0x00,0x22,0x28,0x29,0x00,0x00,0x5b,0x5d,0x7b,
+  0x7d,0x00,0x00,0x00,0x00,0xa2,0xa3,0x00,0x00,0x00,0x00,0x2b,0x2d,0x00,0x00,0x00,
+  0x00,0x3d,0x00,0x3c,0x3e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x5c,
+  0x24,0x00,0x00,0x25,0x23,0x26,0x2a,0x40};
+
+/* Maps multibyte cp932 hiragana/katakana to single-byte equivalents */
+static const unsigned char mbctombb_932_kana[] = {
+  0xa7,0xb1,0xa8,0xb2,0xa9,0xb3,0xaa,0xb4,0xab,0xb5,0xb6,0xb6,0xb7,0xb7,0xb8,0xb8,
+  0xb9,0xb9,0xba,0xba,0xbb,0xbb,0xbc,0xbc,0xbd,0xbd,0xbe,0xbe,0xbf,0xbf,0xc0,0xc0,
+  0xc1,0xc1,0xaf,0xc2,0xc2,0xc3,0xc3,0xc4,0xc4,0xc5,0xc6,0xc7,0xc8,0xc9,0xca,0xca,
+  0xca,0xcb,0xcb,0xcb,0xcc,0xcc,0xcc,0xcd,0xcd,0xcd,0xce,0xce,0xce,0xcf,0xd0,0xd1,
+  0xd2,0xd3,0xac,0xd4,0xad,0xd5,0xae,0xd6,0xd7,0xd8,0xd9,0xda,0xdb,0xdc,0xdc,0xb2,
+  0xb4,0xa6,0xdd,0xb3,0xb6,0xb9};
+
 static MSVCRT_wchar_t msvcrt_mbc_to_wc(unsigned int ch)
 {
   MSVCRT_wchar_t chW;
@@ -74,12 +93,12 @@ static MSVCRT_wchar_t msvcrt_mbc_to_wc(unsigned int ch)
   return chW;
 }
 
-static inline size_t u_strlen( const unsigned char *str )
+static inline MSVCRT_size_t u_strlen( const unsigned char *str )
 {
   return strlen( (const char*) str );
 }
 
-static inline unsigned char* u_strncat( unsigned char* dst, const unsigned char* src, size_t len )
+static inline unsigned char* u_strncat( unsigned char* dst, const unsigned char* src, MSVCRT_size_t len )
 {
   return (unsigned char*)strncat( (char*)dst, (const char*)src, len);
 }
@@ -94,12 +113,12 @@ static inline int u_strcasecmp( const unsigned char *s1, const unsigned char *s2
   return strcasecmp( (const char*)s1, (const char*)s2 );
 }
 
-static inline int u_strncmp( const unsigned char *s1, const unsigned char *s2, size_t len )
+static inline int u_strncmp( const unsigned char *s1, const unsigned char *s2, MSVCRT_size_t len )
 {
   return strncmp( (const char*)s1, (const char*)s2, len );
 }
 
-static inline int u_strncasecmp( const unsigned char *s1, const unsigned char *s2, size_t len )
+static inline int u_strncasecmp( const unsigned char *s1, const unsigned char *s2, MSVCRT_size_t len )
 {
   return strncasecmp( (const char*)s1, (const char*)s2, len );
 }
@@ -124,12 +143,12 @@ static inline unsigned char *u__strset( unsigned char *s, unsigned char c )
   return (unsigned char*) _strset( (char*)s, c);
 }
 
-static inline unsigned char *u__strnset( unsigned char *s, unsigned char c, size_t len )
+static inline unsigned char *u__strnset( unsigned char *s, unsigned char c, MSVCRT_size_t len )
 {
   return (unsigned char*) _strnset( (char*)s, c, len );
 }
 
-static inline size_t u_strcspn( const unsigned char *s, const unsigned char *rej )
+static inline MSVCRT_size_t u_strcspn( const unsigned char *s, const unsigned char *rej )
 {
   return strcspn( (const char *)s, (const char*)rej );
 }
@@ -317,6 +336,34 @@ unsigned int CDECL _mbctoupper(unsigned int c)
     return toupper(c); /* ASCII CP or SB char */
 }
 
+/*********************************************************************
+ *		_mbctombb (MSVCRT.@)
+ */
+unsigned int CDECL _mbctombb(unsigned int c)
+{
+    unsigned int value;
+
+    if(MSVCRT___lc_codepage == 932)
+    {
+        if(c >= 0x829f && c <= 0x82f1)    /* Hiragana */
+            return mbctombb_932_kana[c - 0x829f];
+        if(c >= 0x8340 && c <= 0x8396 && c != 0x837f)    /* Katakana */
+            return mbctombb_932_kana[c - 0x8340 - (c >= 0x837f ? 1 : 0)];
+        if(c >= 0x8140 && c <= 0x8197)    /* Punctuation */
+        {
+            value = mbctombb_932_punct[c - 0x8140];
+            return value ? value : c;
+        }
+        if((c >= 0x824f && c <= 0x8258) || /* Fullwidth digits */
+           (c >= 0x8260 && c <= 0x8279))   /* Fullwidth capitals letters */
+            return c - 0x821f;
+        if(c >= 0x8281 && c <= 0x829a)     /* Fullwidth small letters */
+            return c - 0x8220;
+        /* all other cases return c */
+    }
+    return c;
+}
+
 /*********************************************************************
  *		_mbcjistojms(MSVCRT.@)
  *
@@ -1151,6 +1198,14 @@ int CDECL _ismbbtrail(unsigned int c)
   return (MSVCRT_mbctype[(c&0xff) + 1] & _M2) != 0;
 }
 
+/*********************************************************************
+ *              _ismbclegal(MSVCRT.@)
+ */
+int CDECL _ismbclegal(unsigned int c)
+{
+    return _ismbblead(HIBYTE(c)) && _ismbbtrail(LOBYTE(c));
+}
+
 /*********************************************************************
  *		_ismbslead(MSVCRT.@)
  */
