@@ -7,7 +7,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -50,9 +50,10 @@ static HINSTANCE MSVIDC32_hModule;
 #define CRAM_MAGIC mmioFOURCC('C', 'R', 'A', 'M')
 #define MSVC_MAGIC mmioFOURCC('M', 'S', 'V', 'C')
 #define WHAM_MAGIC mmioFOURCC('W', 'H', 'A', 'M')
+#define compare_fourcc(fcc1, fcc2) (((fcc1)^(fcc2))&~0x20202020)
 
 #define PALETTE_COUNT 256
-#define LE_16(x)  ((((uint8_t*)(x))[1] << 8) | ((uint8_t*)(x))[0])
+#define LE_16(x)  ((((const uint8_t *)(x))[1] << 8) | ((const uint8_t *)(x))[0])
 
 /* FIXME - check the stream size */
 #define CHECK_STREAM_PTR(n) \
@@ -70,7 +71,7 @@ typedef struct Msvideo1Context {
 } Msvideo1Context;
 
 static void 
-msvideo1_decode_8bit( int width, int height, unsigned char *buf, int buf_size,
+msvideo1_decode_8bit( int width, int height, const unsigned char *buf, int buf_size,
                       unsigned char *pixels, int stride)
 {
     int block_ptr, pixel_ptr;
@@ -131,7 +132,7 @@ msvideo1_decode_8bit( int width, int height, unsigned char *buf, int buf_size,
                 for (pixel_y = 0; pixel_y < 4; pixel_y++) {
                     for (pixel_x = 0; pixel_x < 4; pixel_x++, flags >>= 1)
                     {
-#if ORIGINAL
+#ifdef ORIGINAL
                         pixels[pixel_ptr++] = colors[(flags & 0x1) ^ 1];
 #else
                         pixels[width*(height-(pixel_ptr/width)-1) + 
@@ -153,7 +154,7 @@ msvideo1_decode_8bit( int width, int height, unsigned char *buf, int buf_size,
                 for (pixel_y = 0; pixel_y < 4; pixel_y++) {
                     for (pixel_x = 0; pixel_x < 4; pixel_x++, flags >>= 1)
                     {
-#if ORIGINAL
+#ifdef ORIGINAL
                         pixels[pixel_ptr++] = 
                             colors[((pixel_y & 0x2) << 1) + 
                                 (pixel_x & 0x2) + ((flags & 0x1) ^ 1)];
@@ -174,7 +175,7 @@ msvideo1_decode_8bit( int width, int height, unsigned char *buf, int buf_size,
                 for (pixel_y = 0; pixel_y < 4; pixel_y++) {
                     for (pixel_x = 0; pixel_x < 4; pixel_x++)
                     {
-#if ORIGINAL
+#ifdef ORIGINAL
                         pixels[pixel_ptr++] = colors[0];
 #else
                         pixels[width*(height-(pixel_ptr/width)-1) + 
@@ -193,7 +194,7 @@ msvideo1_decode_8bit( int width, int height, unsigned char *buf, int buf_size,
 }
 
 static void
-msvideo1_decode_16bit( int width, int height, unsigned char *buf, int buf_size,
+msvideo1_decode_16bit( int width, int height, const unsigned char *buf, int buf_size,
                        unsigned short *pixels, int stride)
 {
     int block_ptr, pixel_ptr;
@@ -311,9 +312,9 @@ CRAM_DecompressQuery( Msvideo1Context *info, LPBITMAPINFO in, LPBITMAPINFO out )
 
     TRACE("planes = %d\n", in->bmiHeader.biPlanes );
     TRACE("bpp    = %d\n", in->bmiHeader.biBitCount );
-    TRACE("height = %ld\n", in->bmiHeader.biHeight );
-    TRACE("width  = %ld\n", in->bmiHeader.biWidth );
-    TRACE("compr  = %lx\n", in->bmiHeader.biCompression );
+    TRACE("height = %d\n", in->bmiHeader.biHeight );
+    TRACE("width  = %d\n", in->bmiHeader.biWidth );
+    TRACE("compr  = %x\n", in->bmiHeader.biCompression );
 
     if( ( in->bmiHeader.biCompression != CRAM_MAGIC ) &&
         ( in->bmiHeader.biCompression != MSVC_MAGIC ) &&
@@ -397,7 +398,7 @@ static LRESULT CRAM_Decompress( Msvideo1Context *info, ICDECOMPRESS *icd, DWORD
     LONG width, height, stride, sz;
     WORD bit_per_pixel;
 
-    TRACE("ICM_DECOMPRESS %p %p %ld\n", info, icd, size);
+    TRACE("ICM_DECOMPRESS %p %p %d\n", info, icd, size);
 
     if( (info==NULL) || (info->dwMagic!=CRAM_MAGIC) )
         return ICERR_BADPARAM;
@@ -429,7 +430,7 @@ static LRESULT CRAM_DecompressEx( Msvideo1Context *info, ICDECOMPRESSEX *icd, DW
     LONG width, height, stride, sz;
     WORD bit_per_pixel;
 
-    TRACE("ICM_DECOMPRESSEX %p %p %ld\n", info, icd, size);
+    TRACE("ICM_DECOMPRESSEX %p %p %d\n", info, icd, size);
 
     if( (info==NULL) || (info->dwMagic!=CRAM_MAGIC) )
         return ICERR_BADPARAM;
@@ -456,7 +457,7 @@ static LRESULT CRAM_DecompressEx( Msvideo1Context *info, ICDECOMPRESSEX *icd, DW
     return ICERR_OK;
 }
 
-static LRESULT CRAM_GetInfo( Msvideo1Context *info, ICINFO *icinfo, DWORD dwSize )
+static LRESULT CRAM_GetInfo( const Msvideo1Context *info, ICINFO *icinfo, DWORD dwSize )
 {
     if (!icinfo) return sizeof(ICINFO);
     if (dwSize < sizeof(ICINFO)) return 0;
@@ -502,12 +503,12 @@ LRESULT WINAPI CRAM_DriverProc( DWORD_PTR dwDriverId, HDRVR hdrvr, UINT msg,
 
         TRACE("Opened\n");
 
-        if (icinfo && icinfo->fccType != ICTYPE_VIDEO) return 0;
+        if (icinfo && compare_fourcc(icinfo->fccType, ICTYPE_VIDEO)) return 0;
 
         info = HeapAlloc( GetProcessHeap(), 0, sizeof (Msvideo1Context) );
         if( info )
         {
-            memset( info, 0, sizeof info );
+            memset( info, 0, sizeof *info );
             info->dwMagic = CRAM_MAGIC;
         }
         r = (LRESULT) info;
@@ -582,7 +583,7 @@ LRESULT WINAPI CRAM_DriverProc( DWORD_PTR dwDriverId, HDRVR hdrvr, UINT msg,
  */
 BOOL WINAPI DllMain(HINSTANCE hModule, DWORD dwReason, LPVOID lpReserved)
 {
-    TRACE("(%p,%ld,%p)\n", hModule, dwReason, lpReserved);
+    TRACE("(%p,%d,%p)\n", hModule, dwReason, lpReserved);
 
     switch (dwReason)
     {
