@@ -62,7 +62,7 @@ static HRESULT WINAPI domelem_QueryInterface(
 {
     domelem *This = impl_from_IXMLDOMElement( iface );
 
-    TRACE("%p %s %p\n", This, debugstr_guid(riid), ppvObject);
+    TRACE("(%p)->(%s %p)\n", This, debugstr_guid(riid), ppvObject);
 
     if ( IsEqualGUID( riid, &IID_IXMLDOMElement ) ||
          IsEqualGUID( riid, &IID_IDispatch ) ||
@@ -489,32 +489,24 @@ static HRESULT WINAPI domelem_get_tagName(
 {
     domelem *This = impl_from_IXMLDOMElement( iface );
     xmlNodePtr element;
-    DWORD len;
-    DWORD offset = 0;
-    LPWSTR str;
+    const xmlChar *prefix;
+    xmlChar *qname;
 
-    TRACE("%p\n", This );
+    TRACE("(%p)->(%p)\n", This, p );
+
+    if (!p) return E_INVALIDARG;
 
     element = get_element( This );
     if ( !element )
         return E_FAIL;
 
-    len = MultiByteToWideChar( CP_UTF8, 0, (LPCSTR) element->name, -1, NULL, 0 );
-    if (element->ns)
-        len += MultiByteToWideChar( CP_UTF8, 0, (LPCSTR) element->ns->prefix, -1, NULL, 0 );
-    str = heap_alloc( len * sizeof (WCHAR) );
-    if ( !str )
-        return E_OUTOFMEMORY;
-    if (element->ns)
-    {
-        offset = MultiByteToWideChar( CP_UTF8, 0, (LPCSTR) element->ns->prefix, -1, str, len );
-        str[offset - 1] = ':';
-    }
-    MultiByteToWideChar( CP_UTF8, 0, (LPCSTR) element->name, -1, str + offset, len - offset );
-    *p = SysAllocString( str );
-    heap_free( str );
+    prefix = element->ns ? element->ns->prefix : NULL;
+    qname = xmlBuildQName(element->name, prefix, NULL, 0);
 
-    return S_OK;
+    *p = bstr_from_xmlChar(qname);
+    if (qname != element->name) xmlFree(qname);
+
+    return *p ? S_OK : E_OUTOFMEMORY;
 }
 
 static HRESULT WINAPI domelem_getAttribute(
@@ -526,7 +518,7 @@ static HRESULT WINAPI domelem_getAttribute(
     xmlChar *xml_name, *xml_value = NULL;
     HRESULT hr = S_FALSE;
 
-    TRACE("(%p)->(%s,%p)\n", This, debugstr_w(name), value);
+    TRACE("(%p)->(%s %p)\n", This, debugstr_w(name), value);
 
     if(!value || !name)
         return E_INVALIDARG;
@@ -567,7 +559,7 @@ static HRESULT WINAPI domelem_setAttribute(
     HRESULT hr;
     VARIANT var;
 
-    TRACE("(%p)->(%s, var)\n", This, debugstr_w(name));
+    TRACE("(%p)->(%s var)\n", This, debugstr_w(name));
 
     element = get_element( This );
     if ( !element )
@@ -660,7 +652,8 @@ static HRESULT WINAPI domelem_setAttributeNode(
     IXMLDOMAttribute* domAttribute,
     IXMLDOMAttribute** attributeNode)
 {
-    FIXME("\n");
+    domelem *This = impl_from_IXMLDOMElement( iface );
+    FIXME("(%p)->(%p %p)\n", This, domAttribute, attributeNode);
     return E_NOTIMPL;
 }
 
@@ -669,7 +662,8 @@ static HRESULT WINAPI domelem_removeAttributeNode(
     IXMLDOMAttribute* domAttribute,
     IXMLDOMAttribute** attributeNode)
 {
-    FIXME("\n");
+    domelem *This = impl_from_IXMLDOMElement( iface );
+    FIXME("(%p)->(%p %p)\n", This, domAttribute, attributeNode);
     return E_NOTIMPL;
 }
 
@@ -684,7 +678,7 @@ static HRESULT WINAPI domelem_getElementsByTagName(
     xmlNodePtr element;
     HRESULT hr;
 
-    TRACE("(%p)->(%s,%p)\n", This, debugstr_w(bstrName), resultList);
+    TRACE("(%p)->(%s %p)\n", This, debugstr_w(bstrName), resultList);
 
     if (bstrName[0] == '*' && bstrName[1] == 0)
     {
@@ -714,7 +708,8 @@ static HRESULT WINAPI domelem_getElementsByTagName(
 static HRESULT WINAPI domelem_normalize(
     IXMLDOMElement *iface )
 {
-    FIXME("\n");
+    domelem *This = impl_from_IXMLDOMElement( iface );
+    FIXME("%p\n", This);
     return E_NOTIMPL;
 }
 
