@@ -52,6 +52,7 @@ struct relay_descr  /* descriptor for a module */
 };
 
 #define RELAY_DESCR_MAGIC  ((void *)0xdeb90001)
+#define IS_INTARG(x)       (((ULONG_PTR)(x) >> 16) == 0)
 
 /* private data built at dll load time */
 
@@ -147,7 +148,7 @@ static const WCHAR **load_list( HKEY hkey, const WCHAR *value )
     const WCHAR **list = NULL;
 
     RtlInitUnicodeString( &name, value );
-    status = NtQueryValueKey( hkey, &name, KeyValuePartialInformation, buffer, sizeof(buffer), &count );
+    status = NtQueryValueKey( hkey, &name, KeyValuePartialInformation, buffer, sizeof(initial_buffer), &count );
     if (status == STATUS_BUFFER_OVERFLOW)
     {
         buffer = RtlAllocateHeap( GetProcessHeap(), 0, count );
@@ -303,7 +304,7 @@ static inline void RELAY_PrintArgs( const INT_PTR *args, int nb_args, unsigned i
 {
     while (nb_args--)
     {
-	if ((typemask & 3) && HIWORD(*args))
+        if ((typemask & 3) && !IS_INTARG(*args))
         {
 	    if (typemask & 2)
                 DPRINTF( "%08lx %s", *args, debugstr_w((LPCWSTR)*args) );
@@ -320,52 +321,71 @@ static inline void RELAY_PrintArgs( const INT_PTR *args, int nb_args, unsigned i
 extern LONGLONG CDECL call_entry_point( void *func, int nb_args, const INT_PTR *args );
 #ifdef __i386__
 __ASM_GLOBAL_FUNC( call_entry_point,
-                   "\tpushl %ebp\n"
-                   "\tmovl %esp,%ebp\n"
-                   "\tpushl %esi\n"
-                   "\tpushl %edi\n"
-                   "\tmovl 12(%ebp),%edx\n"
-                   "\tshll $2,%edx\n"
-                   "\tjz 1f\n"
-                   "\tsubl %edx,%esp\n"
-                   "\tandl $~15,%esp\n"
-                   "\tmovl 12(%ebp),%ecx\n"
-                   "\tmovl 16(%ebp),%esi\n"
-                   "\tmovl %esp,%edi\n"
-                   "\tcld\n"
-                   "\trep; movsl\n"
-                   "1:\tcall *8(%ebp)\n"
-                   "\tleal -8(%ebp),%esp\n"
-                   "\tpopl %edi\n"
-                   "\tpopl %esi\n"
-                   "\tpopl %ebp\n"
-                   "\tret" )
+                   "pushl %ebp\n\t"
+                   __ASM_CFI(".cfi_adjust_cfa_offset 4\n\t")
+                   __ASM_CFI(".cfi_rel_offset %ebp,0\n\t")
+                   "movl %esp,%ebp\n\t"
+                   __ASM_CFI(".cfi_def_cfa_register %ebp\n\t")
+                   "pushl %esi\n\t"
+                  __ASM_CFI(".cfi_rel_offset %esi,-4\n\t")
+                   "pushl %edi\n\t"
+                  __ASM_CFI(".cfi_rel_offset %edi,-8\n\t")
+                   "movl 12(%ebp),%edx\n\t"
+                   "shll $2,%edx\n\t"
+                   "jz 1f\n\t"
+                   "subl %edx,%esp\n\t"
+                   "andl $~15,%esp\n\t"
+                   "movl 12(%ebp),%ecx\n\t"
+                   "movl 16(%ebp),%esi\n\t"
+                   "movl %esp,%edi\n\t"
+                   "cld\n\t"
+                   "rep; movsl\n"
+                   "1:\tcall *8(%ebp)\n\t"
+                   "leal -8(%ebp),%esp\n\t"
+                   "popl %edi\n\t"
+                   __ASM_CFI(".cfi_same_value %edi\n\t")
+                   "popl %esi\n\t"
+                   __ASM_CFI(".cfi_same_value %esi\n\t")
+                   "popl %ebp\n\t"
+                   __ASM_CFI(".cfi_def_cfa %esp,4\n\t")
+                   __ASM_CFI(".cfi_same_value %ebp\n\t")
+                   "ret" )
 #else
 __ASM_GLOBAL_FUNC( call_entry_point,
-                   "\tpushq %rbp\n"
-                   "\tmovq %rsp,%rbp\n"
-                   "\tpushq %rsi\n"
-                   "\tpushq %rdi\n"
-                   "\tmovq %rcx,%rax\n"
-                   "\tmovq $4,%rcx\n"
-                   "\tcmp %rcx,%rdx\n"
-                   "\tcmovgq %rdx,%rcx\n"
-                   "\tleaq 0(,%rcx,8),%rdx\n"
-                   "\tsubq %rdx,%rsp\n"
-                   "\tandq $~15,%rsp\n"
-                   "\tmovq %rsp,%rdi\n"
-                   "\tmovq %r8,%rsi\n"
-                   "\trep; movsq\n"
-                   "\tmovq 0(%rsp),%rcx\n"
-                   "\tmovq 8(%rsp),%rdx\n"
-                   "\tmovq 16(%rsp),%r8\n"
-                   "\tmovq 24(%rsp),%r9\n"
-                   "\tcallq *%rax\n"
-                   "\tleaq -16(%rbp),%rsp\n"
-                   "\tpopq %rdi\n"
-                   "\tpopq %rsi\n"
-                   "\tpopq %rbp\n"
-                   "\tret\n" )
+                   "pushq %rbp\n\t"
+                   ".cfi_adjust_cfa_offset 8\n\t"
+                   ".cfi_rel_offset %rbp,0\n\t"
+                   "movq %rsp,%rbp\n\t"
+                   ".cfi_def_cfa_register %rbp\n\t"
+                   "pushq %rsi\n\t"
+                   ".cfi_rel_offset %rsi,-8\n\t"
+                   "pushq %rdi\n\t"
+                   ".cfi_rel_offset %rdi,-16\n\t"
+                   "movq %rcx,%rax\n\t"
+                   "movq $4,%rcx\n\t"
+                   "cmp %rcx,%rdx\n\t"
+                   "cmovgq %rdx,%rcx\n\t"
+                   "leaq 0(,%rcx,8),%rdx\n\t"
+                   "subq %rdx,%rsp\n\t"
+                   "andq $~15,%rsp\n\t"
+                   "movq %rsp,%rdi\n\t"
+                   "movq %r8,%rsi\n\t"
+                   "rep; movsq\n\t"
+                   "movq 0(%rsp),%rcx\n\t"
+                   "movq 8(%rsp),%rdx\n\t"
+                   "movq 16(%rsp),%r8\n\t"
+                   "movq 24(%rsp),%r9\n\t"
+                   "callq *%rax\n\t"
+                   "leaq -16(%rbp),%rsp\n\t"
+                   "popq %rdi\n\t"
+                   ".cfi_same_value %rdi\n\t"
+                   "popq %rsi\n\t"
+                   ".cfi_same_value %rsi\n\t"
+                   ".cfi_def_cfa_register %rsp\n\t"
+                   "popq %rbp\n\t"
+                   ".cfi_adjust_cfa_offset -8\n\t"
+                   ".cfi_same_value %rbp\n\t"
+                   "ret")
 #endif
 
 
@@ -861,7 +881,7 @@ static void SNOOP_PrintArg(DWORD x)
     int i,nostring;
 
     DPRINTF("%08x",x);
-    if (!HIWORD(x) || TRACE_ON(seh)) return; /* trivial reject to avoid faults */
+    if (IS_INTARG(x) || TRACE_ON(seh)) return; /* trivial reject to avoid faults */
     __TRY
     {
         LPBYTE s=(LPBYTE)x;
