@@ -591,20 +591,23 @@ static void one_i64toa_test(int test_num, const ulonglong2str_t *ulonglong2str)
     dest_str[LARGE_STRI_BUFFER_LENGTH] = '\0';
     result = p_i64toa(ulonglong2str->value, dest_str, ulonglong2str->base);
     ok(result == dest_str,
-       "(test %d): _i64toa(%Lu, [out], %d) has result %p, expected: %p\n",
-       test_num, ulonglong2str->value, ulonglong2str->base, result, dest_str);
+       "(test %d): _i64toa(%08x%08x, [out], %d) has result %p, expected: %p\n",
+       test_num, (DWORD)(ulonglong2str->value >> 32), (DWORD)ulonglong2str->value,
+       ulonglong2str->base, result, dest_str);
     if (ulonglong2str->mask & 0x04) {
 	if (memcmp(dest_str, ulonglong2str->Buffer, LARGE_STRI_BUFFER_LENGTH) != 0) {
 	    if (memcmp(dest_str, ulonglong2str[1].Buffer, LARGE_STRI_BUFFER_LENGTH) != 0) {
 		ok(memcmp(dest_str, ulonglong2str->Buffer, LARGE_STRI_BUFFER_LENGTH) == 0,
-		   "(test %d): _i64toa(%Lu, [out], %d) assigns string \"%s\", expected: \"%s\"\n",
-		   test_num, ulonglong2str->value, ulonglong2str->base, dest_str, ulonglong2str->Buffer);
+		   "(test %d): _i64toa(%08x%08x, [out], %d) assigns string \"%s\", expected: \"%s\"\n",
+		   test_num, (DWORD)(ulonglong2str->value >> 32), (DWORD)ulonglong2str->value,
+                   ulonglong2str->base, dest_str, ulonglong2str->Buffer);
 	    } /* if */
 	} /* if */
     } else {
 	ok(memcmp(dest_str, ulonglong2str->Buffer, LARGE_STRI_BUFFER_LENGTH) == 0,
-	   "(test %d): _i64toa(%Lu, [out], %d) assigns string \"%s\", expected: \"%s\"\n",
-	   test_num, ulonglong2str->value, ulonglong2str->base, dest_str, ulonglong2str->Buffer);
+	   "(test %d): _i64toa(%08x%08x, [out], %d) assigns string \"%s\", expected: \"%s\"\n",
+	   test_num, (DWORD)(ulonglong2str->value >> 32), (DWORD)ulonglong2str->value,
+           ulonglong2str->base, dest_str, ulonglong2str->Buffer);
     } /* if */
 }
 
@@ -618,11 +621,13 @@ static void one_ui64toa_test(int test_num, const ulonglong2str_t *ulonglong2str)
     dest_str[LARGE_STRI_BUFFER_LENGTH] = '\0';
     result = p_ui64toa(ulonglong2str->value, dest_str, ulonglong2str->base);
     ok(result == dest_str,
-       "(test %d): _ui64toa(%Lu, [out], %d) has result %p, expected: %p\n",
-       test_num, ulonglong2str->value, ulonglong2str->base, result, dest_str);
+       "(test %d): _ui64toa(%08x%08x, [out], %d) has result %p, expected: %p\n",
+       test_num, (DWORD)(ulonglong2str->value >> 32), (DWORD)ulonglong2str->value,
+       ulonglong2str->base, result, dest_str);
     ok(memcmp(dest_str, ulonglong2str->Buffer, LARGE_STRI_BUFFER_LENGTH) == 0,
-       "(test %d): _ui64toa(%Lu, [out], %d) assigns string \"%s\", expected: \"%s\"\n",
-       test_num, ulonglong2str->value, ulonglong2str->base, dest_str, ulonglong2str->Buffer);
+       "(test %d): _ui64toa(%08x%08x, [out], %d) assigns string \"%s\", expected: \"%s\"\n",
+       test_num, (DWORD)(ulonglong2str->value >> 32), (DWORD)ulonglong2str->value,
+       ulonglong2str->base, dest_str, ulonglong2str->Buffer);
 }
 
 
@@ -1108,11 +1113,28 @@ static void test_wtoi64(void)
     }
 }
 
-static void test_wcsfuncs(void)
-{       
-    static const WCHAR testing[] = {'T','e','s','t','i','n','g',0};
-    ok (p_wcschr(testing,0)!=NULL, "wcschr Not finding terminating character\n");
-    ok (p_wcsrchr(testing,0)!=NULL, "wcsrchr Not finding terminating character\n");
+static void test_wcschr(void)
+{
+    static const WCHAR teststringW[] = {'a','b','r','a','c','a','d','a','b','r','a',0};
+
+    ok(p_wcschr(teststringW, 'a') == teststringW + 0,
+       "wcschr should have returned a pointer to the first 'a' character\n");
+    ok(p_wcschr(teststringW, 0) == teststringW + 11,
+       "wcschr should have returned a pointer to the null terminator\n");
+    ok(p_wcschr(teststringW, 'x') == NULL,
+       "wcschr should have returned NULL\n");
+}
+
+static void test_wcsrchr(void)
+{
+    static const WCHAR teststringW[] = {'a','b','r','a','c','a','d','a','b','r','a',0};
+
+    ok(p_wcsrchr(teststringW, 'a') == teststringW + 10,
+       "wcsrchr should have returned a pointer to the last 'a' character\n");
+    ok(p_wcsrchr(teststringW, 0) == teststringW + 11,
+       "wcsrchr should have returned a pointer to the null terminator\n");
+    ok(p_wcsrchr(teststringW, 'x') == NULL,
+       "wcsrchr should have returned NULL\n");
 }
 
 START_TEST(string)
@@ -1135,8 +1157,10 @@ START_TEST(string)
         test_wtol();
     if (p_wtoi64)
         test_wtoi64();
-    if (p_wcschr && p_wcsrchr)
-        test_wcsfuncs();
+    if (p_wcschr)
+        test_wcschr();
+    if (p_wcsrchr)
+        test_wcsrchr();
     if (patoi)
         test_atoi();
     if (patol)
