@@ -40,6 +40,7 @@
 WINE_DEFAULT_DEBUG_CHANNEL(ntoskrnl);
 WINE_DECLARE_DEBUG_CHANNEL(relay);
 
+extern LONG CALLBACK vectored_handler( EXCEPTION_POINTERS *ptrs );
 
 KSYSTEM_TIME KeTickCount = { 0, 0, 0 };
 
@@ -121,27 +122,6 @@ static HANDLE get_device_manager(void)
     return ret;
 }
 
-/* exception handler for emulation of privileged instructions */
-static LONG CALLBACK vectored_handler( EXCEPTION_POINTERS *ptrs )
-{
-    EXCEPTION_RECORD *record = ptrs->ExceptionRecord;
-
-    if (record->ExceptionCode == EXCEPTION_ACCESS_VIOLATION ||
-        record->ExceptionCode == EXCEPTION_PRIV_INSTRUCTION)
-    {
-#ifdef __i386__
-        CONTEXT *context = ptrs->ContextRecord;
-        extern DWORD __wine_emulate_instruction( EXCEPTION_RECORD *rec, CONTEXT *context );
-
-        if (__wine_emulate_instruction( record, context ) == ExceptionContinueExecution)
-            return EXCEPTION_CONTINUE_EXECUTION;
-#else
-        FIXME( "Privileged instruction emulation not implemented on this CPU\n" );
-#endif
-    }
-    return EXCEPTION_CONTINUE_SEARCH;
-}
-
 /* process an ioctl request for a given device */
 static NTSTATUS process_ioctl( DEVICE_OBJECT *device, ULONG code, void *in_buff, ULONG in_size,
                                void *out_buff, ULONG *out_size )
@@ -164,7 +144,7 @@ static NTSTATUS process_ioctl( DEVICE_OBJECT *device, ULONG code, void *in_buff,
     irp.AssociatedIrp.SystemBuffer = in_buff;
     irp.UserBuffer = out_buff;
     irp.MdlAddress = &mdl;
-    irp.Tail.Overlay.s.u.CurrentStackLocation = &irpsp;
+    irp.Tail.Overlay.s.u2.CurrentStackLocation = &irpsp;
     irp.UserIosb = NULL;
 
     irpsp.MajorFunction = IRP_MJ_DEVICE_CONTROL;
@@ -314,7 +294,7 @@ void WINAPI IoInitializeIrp( IRP *irp, USHORT size, CCHAR stack_size )
     InitializeListHead( &irp->ThreadListEntry );
     irp->StackCount = stack_size;
     irp->CurrentLocation = stack_size + 1;
-    irp->Tail.Overlay.s.u.CurrentStackLocation =
+    irp->Tail.Overlay.s.u2.CurrentStackLocation =
             (PIO_STACK_LOCATION)(irp + 1) + stack_size;
 }
 
@@ -454,7 +434,7 @@ PIRP WINAPI IoBuildDeviceIoControlRequest( ULONG IoControlCode,
     instance->irp = irp;
     list_add_tail( &Irps, &instance->entry );
 
-    irpsp = irp->Tail.Overlay.s.u.CurrentStackLocation - 1;
+    irpsp = irp->Tail.Overlay.s.u2.CurrentStackLocation - 1;
     irpsp->MajorFunction = InternalDeviceIoControl ?
             IRP_MJ_INTERNAL_DEVICE_CONTROL : IRP_MJ_DEVICE_CONTROL;
     irpsp->Parameters.DeviceIoControl.IoControlCode = IoControlCode;
@@ -667,7 +647,7 @@ NTSTATUS WINAPI IofCallDriver( DEVICE_OBJECT *device, IRP *irp )
     TRACE( "%p %p\n", device, irp );
 
     --irp->CurrentLocation;
-    irpsp = --irp->Tail.Overlay.s.u.CurrentStackLocation;
+    irpsp = --irp->Tail.Overlay.s.u2.CurrentStackLocation;
     dispatch = device->DriverObject->MajorFunction[irpsp->MajorFunction];
     status = dispatch( device, irp );
 
@@ -764,7 +744,7 @@ void WINAPI IofCompleteRequest( IRP *irp, UCHAR priority_boost )
     status = irp->IoStatus.u.Status;
     while (irp->CurrentLocation <= irp->StackCount)
     {
-        irpsp = irp->Tail.Overlay.s.u.CurrentStackLocation;
+        irpsp = irp->Tail.Overlay.s.u2.CurrentStackLocation;
         routine = irpsp->CompletionRoutine;
         call_flag = 0;
         /* FIXME: add SL_INVOKE_ON_CANCEL support */
@@ -776,7 +756,7 @@ void WINAPI IofCompleteRequest( IRP *irp, UCHAR priority_boost )
                 call_flag = 1;
         }
         ++irp->CurrentLocation;
-        ++irp->Tail.Overlay.s.u.CurrentStackLocation;
+        ++irp->Tail.Overlay.s.u2.CurrentStackLocation;
         if (call_flag)
         {
             TRACE( "calling %p( %p, %p, %p )\n", routine,
@@ -980,6 +960,15 @@ NTSTATUS WINAPI FsRtlRegisterUncProvider(PHANDLE MupHandle, PUNICODE_STRING Redi
     return STATUS_NOT_IMPLEMENTED;
 }
 
+/***********************************************************************
+ *           KeGetCurrentThread / PsGetCurrentThread   (NTOSKRNL.EXE.@)
+ */
+PRKTHREAD WINAPI KeGetCurrentThread(void)
+{
+    FIXME("() stub\n");
+    return NULL;
+}
+
 /***********************************************************************
  *           KeInitializeEvent   (NTOSKRNL.EXE.@)
  */
@@ -998,6 +987,15 @@ void WINAPI KeInitializeMutex(PRKMUTEX Mutex, ULONG Level)
 }
 
 
+/***********************************************************************
+ *           KeInitializeSemaphore   (NTOSKRNL.EXE.@)
+ */
+void WINAPI KeInitializeSemaphore( PRKSEMAPHORE Semaphore, LONG Count, LONG Limit )
+{
+    FIXME( "(%p %d %d) stub\n", Semaphore , Count, Limit );
+}
+
+
 /***********************************************************************
  *           KeInitializeSpinLock   (NTOSKRNL.EXE.@)
  */
@@ -1080,6 +1078,17 @@ void WINAPI KeQueryTickCount( LARGE_INTEGER *count )
 }
 
 
+/***********************************************************************
+ *           KeReleaseSemaphore   (NTOSKRNL.EXE.@)
+ */
+LONG WINAPI KeReleaseSemaphore( PRKSEMAPHORE Semaphore, KPRIORITY Increment,
+                                LONG Adjustment, BOOLEAN Wait )
+{
+    FIXME("(%p %d %d %d) stub\n", Semaphore, Increment, Adjustment, Wait );
+    return 0;
+}
+
+
 /***********************************************************************
  *           KeQueryTimeIncrement   (NTOSKRNL.EXE.@)
  */
@@ -1089,6 +1098,16 @@ ULONG WINAPI KeQueryTimeIncrement(void)
 }
 
 
+/***********************************************************************
+ *           KeSetPriorityThread   (NTOSKRNL.EXE.@)
+ */
+KPRIORITY WINAPI KeSetPriorityThread( PKTHREAD Thread, KPRIORITY Priority )
+{
+    FIXME("(%p %d)\n", Thread, Priority);
+    return Priority;
+}
+
+
 /***********************************************************************
  *           KeWaitForSingleObject   (NTOSKRNL.EXE.@)
  */
@@ -1369,7 +1388,9 @@ BOOL WINAPI DllMain( HINSTANCE inst, DWORD reason, LPVOID reserved )
     {
     case DLL_PROCESS_ATTACH:
         DisableThreadLibraryCalls( inst );
+#ifdef __i386__
         handler = RtlAddVectoredExceptionHandler( TRUE, vectored_handler );
+#endif
         KeQueryTickCount( &count );  /* initialize the global KeTickCount */
         break;
     case DLL_PROCESS_DETACH:
