@@ -37,5 +37,7 @@
 #define DISPID_TM_PROP_WITH_LCID 18
 #define DISPID_TM_PROP_INT 19
 #define DISPID_TM_PROP_UINT 20
+#define DISPID_TM_BYREF_UINT 21
 
 #define DISPID_NOA_BSTRRET 1
+#define DISPID_NOA_ERROR 2
