@@ -370,6 +370,7 @@ static BOOL do_reg_operation( HKEY hkey, const WCHAR *value, INFCONTEXT *context
             {
                 if (!(str = HeapAlloc( GetProcessHeap(), 0, size * sizeof(WCHAR) ))) return FALSE;
                 SetupGetStringFieldW( context, 5, str, size, NULL );
+                if (type == REG_LINK) size--;  /* no terminating null for symlinks */
             }
         }
 
@@ -421,6 +422,7 @@ static BOOL registry_callback( HINF hinf, PCWSTR field, void *arg )
 
     for (; ok; ok = SetupFindNextLine( &context, &context ))
     {
+        DWORD options = 0;
         WCHAR buffer[MAX_INF_STRING_LENGTH];
         INT flags;
 
@@ -446,15 +448,26 @@ static BOOL registry_callback( HINF hinf, PCWSTR field, void *arg )
             if (!flags) flags = FLG_ADDREG_DELREG_BIT;
             else if (!(flags & FLG_ADDREG_DELREG_BIT)) continue;  /* ignore this entry */
         }
+        /* Wine extension: magic support for symlinks */
+        if (flags >> 16 == REG_LINK) options = REG_OPTION_OPEN_LINK | REG_OPTION_CREATE_LINK;
 
         if (info->delete || (flags & FLG_ADDREG_OVERWRITEONLY))
         {
-            if (RegOpenKeyW( root_key, buffer, &hkey )) continue;  /* ignore if it doesn't exist */
+            if (RegOpenKeyExW( root_key, buffer, options, MAXIMUM_ALLOWED, &hkey ))
+                continue;  /* ignore if it doesn't exist */
         }
-        else if (RegCreateKeyW( root_key, buffer, &hkey ))
+        else
         {
-            ERR( "could not create key %p %s\n", root_key, debugstr_w(buffer) );
-            continue;
+            DWORD res = RegCreateKeyExW( root_key, buffer, 0, NULL, options,
+                                         MAXIMUM_ALLOWED, NULL, &hkey, NULL );
+            if (res == ERROR_ALREADY_EXISTS && (options & REG_OPTION_CREATE_LINK))
+                res = RegCreateKeyExW( root_key, buffer, 0, NULL, REG_OPTION_OPEN_LINK,
+                                       MAXIMUM_ALLOWED, NULL, &hkey, NULL );
+            if (res)
+            {
+                ERR( "could not create key %p %s\n", root_key, debugstr_w(buffer) );
+                continue;
+            }
         }
         TRACE( "key %p %s\n", root_key, debugstr_w(buffer) );
 
@@ -700,6 +713,7 @@ static BOOL fake_dlls_callback( HINF hinf, PCWSTR field, void *arg )
         HeapFree( GetProcessHeap(), 0, path );
         if (!ret) break;
     }
+    cleanup_fake_dlls();
     return ret;
 }
 
@@ -787,9 +801,6 @@ static BOOL profile_items_callback( HINF hinf, PCWSTR field, void *arg )
     LPWSTR cmdline=NULL, lnkpath_end;
     unsigned int name_size;
     INFCONTEXT name_context, context;
-    IShellLinkW* shelllink=NULL;
-    IPersistFile* persistfile=NULL;
-    HRESULT initresult=E_FAIL;
     int attrs=0;
 
     static const WCHAR dotlnk[] = {'.','l','n','k',0};
@@ -799,7 +810,7 @@ static BOOL profile_items_callback( HINF hinf, PCWSTR field, void *arg )
     if (SetupFindFirstLineW( hinf, field, Name, &name_context ))
     {
         SetupGetIntField( &name_context, 2, &attrs );
-        if (attrs) FIXME( "unhandled attributes: %x\n", attrs );
+        if (attrs & ~FLG_PROFITEM_GROUP) FIXME( "unhandled attributes: %x\n", attrs );
     }
     else return TRUE;
 
@@ -808,7 +819,7 @@ static BOOL profile_items_callback( HINF hinf, PCWSTR field, void *arg )
     lnkpath_end = lnkpath + strlenW(lnkpath);
     if (lnkpath_end[-1] != '\\') *lnkpath_end++ = '\\';
 
-    if (SetupFindFirstLineW( hinf, field, SubDir, &context ))
+    if (!(attrs & FLG_PROFITEM_GROUP) && SetupFindFirstLineW( hinf, field, SubDir, &context ))
     {
         unsigned int subdir_size;
 
@@ -823,67 +834,80 @@ static BOOL profile_items_callback( HINF hinf, PCWSTR field, void *arg )
         return TRUE;
 
     lnkpath_end += name_size - 1;
-    if (lnkpath+MAX_PATH < lnkpath_end + 5) return TRUE;
-    strcpyW( lnkpath_end, dotlnk );
 
-    TRACE( "link path: %s\n", debugstr_w(lnkpath) );
-
-    /* calculate command line */
-    if (SetupFindFirstLineW( hinf, field, CmdLine, &context ))
+    if (attrs & FLG_PROFITEM_GROUP)
     {
-        unsigned int dir_len=0, subdir_size=0, filename_size=0;
-        int dirid=0;
-        LPCWSTR dir;
-        LPWSTR cmdline_end;
+        SHPathPrepareForWriteW( NULL, NULL, lnkpath, SHPPFW_DIRCREATE );
+    }
+    else
+    {
+        IShellLinkW* shelllink=NULL;
+        IPersistFile* persistfile=NULL;
+        HRESULT initresult=E_FAIL;
 
-        SetupGetIntField( &context, 1, &dirid );
-        dir = DIRID_get_string( dirid );
+        if (lnkpath+MAX_PATH < lnkpath_end + 5) return TRUE;
+        strcpyW( lnkpath_end, dotlnk );
 
-        if (dir) dir_len = strlenW(dir);
+        TRACE( "link path: %s\n", debugstr_w(lnkpath) );
 
-        SetupGetStringFieldW( &context, 2, NULL, 0, &subdir_size );
-        SetupGetStringFieldW( &context, 3, NULL, 0, &filename_size );
-
-        if (dir_len && filename_size)
+        /* calculate command line */
+        if (SetupFindFirstLineW( hinf, field, CmdLine, &context ))
         {
-            cmdline = cmdline_end = HeapAlloc( GetProcessHeap(), 0, sizeof(WCHAR) * (dir_len+subdir_size+filename_size+1) );
+            unsigned int dir_len=0, subdir_size=0, filename_size=0;
+            int dirid=0;
+            LPCWSTR dir;
+            LPWSTR cmdline_end;
 
-            strcpyW( cmdline_end, dir );
-            cmdline_end += dir_len;
-            if (cmdline_end[-1] != '\\') *cmdline_end++ = '\\';
+            SetupGetIntField( &context, 1, &dirid );
+            dir = DIRID_get_string( dirid );
 
-            if (subdir_size)
+            if (dir) dir_len = strlenW(dir);
+
+            SetupGetStringFieldW( &context, 2, NULL, 0, &subdir_size );
+            SetupGetStringFieldW( &context, 3, NULL, 0, &filename_size );
+
+            if (dir_len && filename_size)
             {
-                SetupGetStringFieldW( &context, 2, cmdline_end, subdir_size, NULL );
-                cmdline_end += subdir_size-1;
+                cmdline = cmdline_end = HeapAlloc( GetProcessHeap(), 0, sizeof(WCHAR) * (dir_len+subdir_size+filename_size+1) );
+
+                strcpyW( cmdline_end, dir );
+                cmdline_end += dir_len;
                 if (cmdline_end[-1] != '\\') *cmdline_end++ = '\\';
+
+                if (subdir_size)
+                {
+                    SetupGetStringFieldW( &context, 2, cmdline_end, subdir_size, NULL );
+                    cmdline_end += subdir_size-1;
+                    if (cmdline_end[-1] != '\\') *cmdline_end++ = '\\';
+                }
+                SetupGetStringFieldW( &context, 3, cmdline_end, filename_size, NULL );
+                TRACE( "cmdline: %s\n", debugstr_w(cmdline));
             }
-            SetupGetStringFieldW( &context, 3, cmdline_end, filename_size, NULL );
-            TRACE( "cmdline: %s\n", debugstr_w(cmdline));
         }
+
+        if (!cmdline) return TRUE;
+
+        initresult = CoInitialize(NULL);
+
+        if (FAILED(CoCreateInstance( &CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER,
+                                     &IID_IShellLinkW, (LPVOID*)&shelllink )))
+            goto done;
+
+        IShellLinkW_SetPath( shelllink, cmdline );
+        SHPathPrepareForWriteW( NULL, NULL, lnkpath, SHPPFW_DIRCREATE|SHPPFW_IGNOREFILENAME );
+        if (SUCCEEDED(IShellLinkW_QueryInterface( shelllink, &IID_IPersistFile, (LPVOID*)&persistfile)))
+        {
+            TRACE( "writing link: %s\n", debugstr_w(lnkpath) );
+            IPersistFile_Save( persistfile, lnkpath, FALSE );
+            IPersistFile_Release( persistfile );
+        }
+        IShellLinkW_Release( shelllink );
+
+    done:
+        if (SUCCEEDED(initresult)) CoUninitialize();
+        HeapFree( GetProcessHeap(), 0, cmdline );
     }
 
-    if (!cmdline) return TRUE;
-
-    initresult = CoInitialize(NULL);
-
-    if (!SUCCEEDED(CoCreateInstance( &CLSID_ShellLink, NULL,
-                                     CLSCTX_INPROC_SERVER, &IID_IShellLinkW, (LPVOID*)&shelllink)))
-        goto done;
-
-    IShellLinkW_SetPath( shelllink, cmdline );
-    SHPathPrepareForWriteW( NULL, NULL, lnkpath, SHPPFW_DIRCREATE|SHPPFW_IGNOREFILENAME );
-    if (SUCCEEDED(IShellLinkW_QueryInterface( shelllink, &IID_IPersistFile, (LPVOID*)&persistfile)))
-    {
-        TRACE( "writing link: %s\n", debugstr_w(lnkpath) );
-        IPersistFile_Save( persistfile, lnkpath, FALSE );
-        IPersistFile_Release( persistfile );
-    }
-    IShellLinkW_Release( shelllink );
-
-done:
-    if (SUCCEEDED(initresult)) CoUninitialize();
-    HeapFree( GetProcessHeap(), 0, cmdline );
     return TRUE;
 }
 
@@ -1071,10 +1095,10 @@ BOOL WINAPI SetupInstallFromInfSectionW( HWND owner, HINF hinf, PCWSTR section,
         }
         else info.callback = NULL;
 
-        if (!iterate_section_fields( hinf, section, RegisterDlls, register_dlls_callback, &info ))
+        if (!iterate_section_fields( hinf, section, WineFakeDlls, fake_dlls_callback, NULL ))
             return FALSE;
 
-        if (!iterate_section_fields( hinf, section, WineFakeDlls, fake_dlls_callback, NULL ))
+        if (!iterate_section_fields( hinf, section, RegisterDlls, register_dlls_callback, &info ))
             return FALSE;
     }
     if (flags & SPINST_UNREGSVR)
@@ -1435,3 +1459,141 @@ BOOL WINAPI SetupInstallServicesFromInfSectionA( HINF Inf, PCSTR SectionName, DW
 
     return ret;
 }
+
+
+/***********************************************************************
+ *              SetupGetInfFileListW  (SETUPAPI.@)
+ */
+BOOL WINAPI SetupGetInfFileListW(PCWSTR dir, DWORD style, PWSTR buffer,
+                                 DWORD insize, PDWORD outsize)
+{
+    static WCHAR inf[] = {'\\','*','.','i','n','f',0 };
+    WCHAR *filter, *fullname = NULL, *ptr = buffer;
+    DWORD dir_len, name_len = 20, size ;
+    WIN32_FIND_DATAW finddata;
+    HANDLE hdl;
+    if (style & ~( INF_STYLE_OLDNT | INF_STYLE_WIN4 |
+                   INF_STYLE_CACHE_ENABLE | INF_STYLE_CACHE_DISABLE ))
+    {
+        FIXME( "unknown inf_style(s) 0x%x\n",
+               style & ~( INF_STYLE_OLDNT | INF_STYLE_WIN4 |
+                         INF_STYLE_CACHE_ENABLE | INF_STYLE_CACHE_DISABLE ));
+        if( outsize ) *outsize = 1;
+        return TRUE;
+    }
+    if ((style & ( INF_STYLE_OLDNT | INF_STYLE_WIN4 )) == INF_STYLE_NONE)
+    {
+        FIXME( "inf_style INF_STYLE_NONE not handled\n" );
+        if( outsize ) *outsize = 1;
+        return TRUE;
+    }
+    if (style & ( INF_STYLE_CACHE_ENABLE | INF_STYLE_CACHE_DISABLE ))
+        FIXME("ignored inf_style(s) %s %s\n",
+              ( style & INF_STYLE_CACHE_ENABLE  ) ? "INF_STYLE_CACHE_ENABLE"  : "",
+              ( style & INF_STYLE_CACHE_DISABLE ) ? "INF_STYLE_CACHE_DISABLE" : "");
+    if( dir )
+    {
+        DWORD att;
+        DWORD msize;
+        dir_len = strlenW( dir );
+        if ( !dir_len ) return FALSE;
+        msize = ( 7 + dir_len )  * sizeof( WCHAR ); /* \\*.inf\0 */
+        filter = HeapAlloc( GetProcessHeap(), 0, msize );
+        if( !filter )
+        {
+            SetLastError( ERROR_NOT_ENOUGH_MEMORY );
+            return FALSE;
+        }
+        strcpyW( filter, dir );
+        if ( '\\' == filter[dir_len - 1] )
+            filter[--dir_len] = 0;
+
+        att = GetFileAttributesW( filter );
+        if (att != INVALID_FILE_ATTRIBUTES && !(att & FILE_ATTRIBUTE_DIRECTORY))
+        {
+            HeapFree( GetProcessHeap(), 0, filter );
+            SetLastError( ERROR_DIRECTORY );
+            return FALSE;
+        }
+    }
+    else
+    {
+        WCHAR infdir[] = {'\\','i','n','f',0 };
+        DWORD msize;
+        dir_len = GetWindowsDirectoryW( NULL, 0 );
+        msize = ( 7 + 4 + dir_len ) * sizeof( WCHAR );
+        filter = HeapAlloc( GetProcessHeap(), 0, msize );
+        if( !filter )
+        {
+            SetLastError( ERROR_NOT_ENOUGH_MEMORY );
+            return FALSE;
+        }
+        GetWindowsDirectoryW( filter, msize );
+        strcatW( filter, infdir );
+    }
+    strcatW( filter, inf );
+
+    hdl = FindFirstFileW( filter , &finddata );
+    if ( hdl == INVALID_HANDLE_VALUE )
+    {
+        if( outsize ) *outsize = 1;
+        HeapFree( GetProcessHeap(), 0, filter );
+        return TRUE;
+    }
+    size = 1;
+    do
+    {
+        static const WCHAR key[] =
+               {'S','i','g','n','a','t','u','r','e',0 };
+        static const WCHAR section[] =
+               {'V','e','r','s','i','o','n',0 };
+        static const WCHAR sig_win4_1[] =
+               {'$','C','h','i','c','a','g','o','$',0 };
+        static const WCHAR sig_win4_2[] =
+               {'$','W','I','N','D','O','W','S',' ','N','T','$',0 };
+        WCHAR signature[ MAX_PATH ];
+        BOOL valid = FALSE;
+        DWORD len = strlenW( finddata.cFileName );
+        if (!fullname || ( name_len < len ))
+        {
+            name_len = ( name_len < len ) ? len : name_len;
+            HeapFree( GetProcessHeap(), 0, fullname );
+            fullname = HeapAlloc( GetProcessHeap(), 0,
+                                  ( 2 + dir_len + name_len) * sizeof( WCHAR ));
+            if( !fullname )
+            {
+                HeapFree( GetProcessHeap(), 0, filter );
+                SetLastError( ERROR_NOT_ENOUGH_MEMORY );
+                return FALSE;
+            }
+            strcpyW( fullname, filter );
+        }
+        fullname[ dir_len + 1] = 0; /* keep '\\' */
+        strcatW( fullname, finddata.cFileName );
+        if (!GetPrivateProfileStringW( section, key, NULL, signature, MAX_PATH, fullname ))
+            signature[0] = 0;
+        if( INF_STYLE_OLDNT & style )
+            valid = strcmpiW( sig_win4_1, signature ) &&
+                    strcmpiW( sig_win4_2, signature );
+        if( INF_STYLE_WIN4 & style )
+            valid = valid || !strcmpiW( sig_win4_1, signature ) ||
+                    !strcmpiW( sig_win4_2, signature );
+        if( valid )
+        {
+            size += 1 + strlenW( finddata.cFileName );
+            if( ptr && insize >= size )
+            {
+                strcpyW( ptr, finddata.cFileName );
+                ptr += 1 + strlenW( finddata.cFileName );
+                *ptr = 0;
+            }
+        }
+    }
+    while( FindNextFileW( hdl, &finddata ));
+    FindClose( hdl );
+
+    HeapFree( GetProcessHeap(), 0, fullname );
+    HeapFree( GetProcessHeap(), 0, filter );
+    if( outsize ) *outsize = size;
+    return TRUE;
+}
