@@ -39,7 +39,7 @@
 #include "wine/debug.h"
 #include "wine/unicode.h"
 
-WINE_DEFAULT_DEBUG_CHANNEL(ole);
+WINE_DEFAULT_DEBUG_CHANNEL(shell);
 
 /*
  * Near the bottom of this file are the exported DllRegisterServer and
@@ -658,6 +658,13 @@ static struct regsvr_coclass const coclass_list[] = {
 	"shell32.dll",
 	"Apartment"
     },
+    {   &CLSID_QueryAssociations,
+        "Query file associations",
+        0,
+        NULL,
+        "shell32.dll",
+        "Apartment"
+    },
     {   &CLSID_ControlPanel,
         "Control Panel",
         IDS_CONTROLPANEL,
