@@ -303,7 +303,10 @@ static ULONG ShellLink_Release( IShellLinkImpl *This )
     HeapFree(GetProcessHeap(), 0, This->sArgs);
     HeapFree(GetProcessHeap(), 0, This->sWorkDir);
     HeapFree(GetProcessHeap(), 0, This->sDescription);
-    HeapFree(GetProcessHeap(),0,This->sPath);
+    HeapFree(GetProcessHeap(), 0, This->sPath);
+    HeapFree(GetProcessHeap(), 0, This->sPathRel);
+    HeapFree(GetProcessHeap(), 0, This->sProduct);
+    HeapFree(GetProcessHeap(), 0, This->sComponent);
 
     if (This->site)
         IUnknown_Release( This->site );
@@ -496,7 +499,7 @@ static HRESULT WINAPI IPersistFile_fnSaveCompleted(IPersistFile* iface, LPCOLEST
 static HRESULT WINAPI IPersistFile_fnGetCurFile(IPersistFile* iface, LPOLESTR *ppszFileName)
 {
 	IShellLinkImpl *This = impl_from_IPersistFile(iface);
-	FIXME("(%p)\n",This);
+	FIXME("(%p)->(%p): stub\n", This, ppszFileName);
 	return NOERROR;
 }
 
@@ -1497,9 +1500,13 @@ static HRESULT WINAPI IShellLinkA_fnSetArguments(IShellLinkA * iface, LPCSTR psz
     TRACE("(%p)->(args=%s)\n",This, pszArgs);
 
     HeapFree(GetProcessHeap(), 0, This->sArgs);
-    This->sArgs = HEAP_strdupAtoW( GetProcessHeap(), 0, pszArgs);
-    if( !This->sArgs )
-        return E_OUTOFMEMORY;
+    if (pszArgs)
+    {
+        This->sArgs = HEAP_strdupAtoW( GetProcessHeap(), 0, pszArgs);
+        if( !This->sArgs )
+            return E_OUTOFMEMORY;
+    }
+    else This->sArgs = NULL;
 
     This->bDirty = TRUE;
 
@@ -1877,11 +1884,16 @@ static HRESULT WINAPI IShellLinkW_fnSetArguments(IShellLinkW * iface, LPCWSTR ps
     TRACE("(%p)->(args=%s)\n",This, debugstr_w(pszArgs));
 
     HeapFree(GetProcessHeap(), 0, This->sArgs);
-    This->sArgs = HeapAlloc( GetProcessHeap(), 0,
-                             (lstrlenW( pszArgs )+1)*sizeof (WCHAR) );
-    if ( !This->sArgs )
-        return E_OUTOFMEMORY;
-    lstrcpyW( This->sArgs, pszArgs );
+    if (pszArgs)
+    {
+        This->sArgs = HeapAlloc( GetProcessHeap(), 0,
+                                 (lstrlenW( pszArgs )+1)*sizeof (WCHAR) );
+        if ( !This->sArgs )
+            return E_OUTOFMEMORY;
+        lstrcpyW( This->sArgs, pszArgs );
+    }
+    else This->sArgs = NULL;
+
     This->bDirty = TRUE;
 
     return S_OK;
@@ -2204,7 +2216,10 @@ static HRESULT WINAPI IShellLinkW_fnSetPath(IShellLinkW * iface, LPCWSTR pszFile
 
     /* any other quote marks are invalid */
     if (strchrW(pszFile, '"'))
+    {
+        HeapFree(GetProcessHeap(), 0, unquoted);
         return S_FALSE;
+    }
 
     HeapFree(GetProcessHeap(), 0, This->sPath);
     This->sPath = NULL;
@@ -2232,7 +2247,10 @@ static HRESULT WINAPI IShellLinkW_fnSetPath(IShellLinkW * iface, LPCWSTR pszFile
         This->sPath = HeapAlloc( GetProcessHeap(), 0,
                              (lstrlenW( buffer )+1) * sizeof (WCHAR) );
         if (!This->sPath)
+        {
+            HeapFree(GetProcessHeap(), 0, unquoted);
             return E_OUTOFMEMORY;
+        }
 
         lstrcpyW(This->sPath, buffer);
     }
@@ -2295,7 +2313,7 @@ ShellLink_DataList_Release( IShellLinkDataList* iface )
 static HRESULT WINAPI
 ShellLink_AddDataBlock( IShellLinkDataList* iface, void* pDataBlock )
 {
-    FIXME("\n");
+    FIXME("(%p)->(%p): stub\n", iface, pDataBlock);
     return E_NOTIMPL;
 }
 
@@ -2333,7 +2351,7 @@ ShellLink_CopyDataBlock( IShellLinkDataList* iface, DWORD dwSig, void** ppDataBl
 static HRESULT WINAPI
 ShellLink_RemoveDataBlock( IShellLinkDataList* iface, DWORD dwSig )
 {
-    FIXME("\n");
+    FIXME("(%p)->(%u): stub\n", iface, dwSig);
     return E_NOTIMPL;
 }
 
@@ -2343,7 +2361,7 @@ ShellLink_GetFlags( IShellLinkDataList* iface, DWORD* pdwFlags )
     IShellLinkImpl *This = impl_from_IShellLinkDataList(iface);
     DWORD flags = 0;
 
-    FIXME("%p %p\n", This, pdwFlags );
+    FIXME("(%p)->(%p): partially implemented\n", This, pdwFlags);
 
     /* FIXME: add more */
     if (This->sArgs)
@@ -2365,7 +2383,7 @@ ShellLink_GetFlags( IShellLinkDataList* iface, DWORD* pdwFlags )
 static HRESULT WINAPI
 ShellLink_SetFlags( IShellLinkDataList* iface, DWORD dwFlags )
 {
-    FIXME("\n");
+    FIXME("(%p)->(%u): stub\n", iface, dwFlags);
     return E_NOTIMPL;
 }
 
@@ -2621,7 +2639,7 @@ ShellLink_GetCommandString( IContextMenu* iface, UINT_PTR idCmd, UINT uType,
 {
     IShellLinkImpl *This = impl_from_IContextMenu(iface);
 
-    FIXME("%p %lu %u %p %p %u\n", This,
+    FIXME("(%p)->(%lu %u %p %p %u): stub\n", This,
           idCmd, uType, pwReserved, pszName, cchMax );
 
     return E_NOTIMPL;
