@@ -64,6 +64,7 @@ static const struct {
 	{&CLSID_MyComputer,	ISF_MyComputer_Constructor},
 	{&CLSID_NetworkPlaces,  ISF_NetworkPlaces_Constructor},
 	{&CLSID_ShellDesktop,	ISF_Desktop_Constructor},
+	{&CLSID_ShellItem,	IShellItem_Constructor},
 	{&CLSID_ShellLink,	IShellLink_Constructor},
 	{&CLSID_DragDropHelper, IDropTargetHelper_Constructor},
 	{&CLSID_ControlPanel,	IControlPanel_Constructor},
@@ -73,6 +74,7 @@ static const struct {
 	{&CLSID_FolderShortcut, FolderShortcut_Constructor},
 	{&CLSID_MyDocuments,    MyDocuments_Constructor},
 	{&CLSID_RecycleBin,     RecycleBin_Constructor},
+	{&CLSID_QueryAssociations, QueryAssociations_Constructor},
 	{NULL,NULL}
 };
 
@@ -518,7 +520,7 @@ void WINAPI DragAcceptFiles(HWND hWnd, BOOL b)
 void WINAPI DragFinish(HDROP h)
 {
 	TRACE("\n");
-	GlobalFree((HGLOBAL)h);
+	GlobalFree(h);
 }
 
 /*************************************************************************
