@@ -57,6 +57,7 @@
 #include "wingdi.h"
 #include "winuser.h"
 #include "shlobj.h"
+#include "shobjidl.h"
 #include "undocshell.h"
 #include "shresdef.h"
 #include "wine/debug.h"
@@ -84,6 +85,7 @@ typedef struct
 	const IDropTargetVtbl*	lpvtblDropTarget;
 	const IDropSourceVtbl*	lpvtblDropSource;
 	const IViewObjectVtbl*	lpvtblViewObject;
+	const IFolderViewVtbl*  lpvtblFolderView;
 	IShellFolder*	pSFParent;
 	IShellFolder2*	pSF2Parent;
 	IShellBrowser*	pShellBrowser;
@@ -114,7 +116,7 @@ static const IOleCommandTargetVtbl ctvt;
 static const IDropTargetVtbl dtvt;
 static const IDropSourceVtbl dsvt;
 static const IViewObjectVtbl vovt;
-
+static const IFolderViewVtbl fviewvt;
 
 static inline IShellViewImpl *impl_from_IOleCommandTarget( IOleCommandTarget *iface )
 {
@@ -136,6 +138,11 @@ static inline IShellViewImpl *impl_from_IViewObject( IViewObject *iface )
     return (IShellViewImpl *)((char*)iface - FIELD_OFFSET(IShellViewImpl, lpvtblViewObject));
 }
 
+static inline IShellViewImpl *impl_from_IFolderView( IFolderView *iface )
+{
+    return (IShellViewImpl *)((char*)iface - FIELD_OFFSET(IShellViewImpl, lpvtblFolderView));
+}
+
 /* ListView Header ID's */
 #define LISTVIEW_COLUMN_NAME 0
 #define LISTVIEW_COLUMN_SIZE 1
@@ -192,6 +199,7 @@ IShellView * IShellView_Constructor( IShellFolder * pFolder)
 	sv->lpvtblDropTarget=&dtvt;
 	sv->lpvtblDropSource=&dsvt;
 	sv->lpvtblViewObject=&vovt;
+	sv->lpvtblFolderView=&fviewvt;
 
 	sv->pSFParent = pFolder;
 	if(pFolder) IShellFolder_AddRef(pFolder);
@@ -560,7 +568,7 @@ static BOOLEAN LV_AddItem(IShellViewImpl * This, LPCITEMIDLIST pidl)
 	TRACE("(%p)(pidl=%p)\n", This, pidl);
 
 	lvItem.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM;	/*set the mask*/
-	lvItem.iItem = ListView_GetItemCount(This->hWndList);	/*add the item to the end of the list*/
+	lvItem.iItem = SendMessageW(This->hWndList, LVM_GETITEMCOUNT, 0, 0); /*add the item to the end of the list*/
 	lvItem.iSubItem = 0;
 	lvItem.lParam = (LPARAM) ILClone(ILFindLastID(pidl));				/*set the item's data*/
 	lvItem.pszText = LPSTR_TEXTCALLBACKW;			/*get text on a callback basis*/
@@ -578,7 +586,7 @@ static BOOLEAN LV_DeleteItem(IShellViewImpl * This, LPCITEMIDLIST pidl)
 	TRACE("(%p)(pidl=%p)\n", This, pidl);
 
 	nIndex = LV_FindItemByPidl(This, ILFindLastID(pidl));
-	return (-1==ListView_DeleteItem(This->hWndList, nIndex))? FALSE: TRUE;
+	return (-1==SendMessageW(This->hWndList, LVM_DELETEITEM, nIndex, 0))? FALSE: TRUE;
 }
 
 /**********************************************************
@@ -836,7 +844,7 @@ static UINT ShellView_GetSelections(IShellViewImpl * This)
 
 	SHFree(This->apidl);
 
-	This->cidl = ListView_GetSelectedCount(This->hWndList);
+	This->cidl = SendMessageW(This->hWndList, LVM_GETSELECTEDCOUNT, 0, 0);
 	This->apidl = SHAlloc(This->cidl * sizeof(LPITEMIDLIST));
 
 	TRACE("selected=%i\n", This->cidl);
@@ -976,7 +984,7 @@ static void ShellView_DoContextMenu(IShellViewImpl * This, WORD x, WORD y, BOOL
 	if( ShellView_GetSelections(This) )
 	{
 	  IShellFolder_GetUIObjectOf( This->pSFParent, This->hWndParent, This->cidl, (LPCITEMIDLIST*)This->apidl,
-					(REFIID)&IID_IContextMenu, NULL, (LPVOID *)&pContextMenu);
+                                      &IID_IContextMenu, NULL, (LPVOID *)&pContextMenu);
 
 	  if(pContextMenu)
 	  {
@@ -1262,7 +1270,7 @@ static LRESULT ShellView_OnCommand(IShellViewImpl * This,DWORD dwCmdID, DWORD dw
 	  case 0x31:
 	  case 0x32:
 	  case 0x33:
-	    This->ListViewSortInfo.nHeaderID = (LPARAM) (dwCmdID - 0x30);
+            This->ListViewSortInfo.nHeaderID = dwCmdID - 0x30;
 	    This->ListViewSortInfo.bIsAscending = TRUE;
 	    This->ListViewSortInfo.nLastHeaderID = This->ListViewSortInfo.nHeaderID;
 	    SendMessageA(This->hWndList, LVM_SORTITEMS, (WPARAM) &This->ListViewSortInfo, (LPARAM)ShellView_ListViewCompareItems);
@@ -1321,7 +1329,7 @@ static LRESULT ShellView_OnNotify(IShellViewImpl * This, UINT CtlID, LPNMHDR lpn
             break;
 
           case NM_RETURN:
-            TRACE("-- NM_DBLCLK %p\n",This);
+            TRACE("-- NM_RETURN %p\n",This);
             if (OnDefaultCommand(This) != S_OK) ShellView_OpenSelectedItems(This);
             break;
 
@@ -1428,8 +1436,8 @@ static LRESULT ShellView_OnNotify(IShellViewImpl * This, UINT CtlID, LPNMHDR lpn
 
 	          if (pds)
 	          {
-	            DWORD dwEffect;
-		    DoDragDrop(pda, pds, dwEffect, &dwEffect);
+	            DWORD dwEffect2;
+		    DoDragDrop(pda, pds, dwEffect, &dwEffect2);
 		  }
 	          IDataObject_Release(pda);
 	      }
@@ -1441,7 +1449,7 @@ static LRESULT ShellView_OnNotify(IShellViewImpl * This, UINT CtlID, LPNMHDR lpn
 	      DWORD dwAttr = SFGAO_CANRENAME;
 	      pidl = (LPITEMIDLIST)lpdi->item.lParam;
 
-	      TRACE("-- LVN_BEGINLABELEDITA %p\n",This);
+	      TRACE("-- LVN_BEGINLABELEDITW %p\n",This);
 
 	      IShellFolder_GetAttributesOf(This->pSFParent, 1, (LPCITEMIDLIST*)&pidl, &dwAttr);
 	      if (SFGAO_CANRENAME & dwAttr)
@@ -1494,14 +1502,13 @@ static LRESULT ShellView_OnNotify(IShellViewImpl * This, UINT CtlID, LPNMHDR lpn
               if(plvKeyDown->wVKey == VK_F2)
               {
                 /* see how many files are selected */
-                int i = ListView_GetSelectedCount(This->hWndList);
+                int i = SendMessageW(This->hWndList, LVM_GETSELECTEDCOUNT, 0, 0);
 
                 /* get selected item */
                 if(i == 1)
                 {
                   /* get selected item */
-                  i = ListView_GetNextItem(This->hWndList, -1,
-			LVNI_SELECTED);
+                  i = SendMessageW(This->hWndList, LVM_GETNEXTITEM, -1, MAKELPARAM (LVNI_SELECTED, 0));
 
                   SendMessageW(This->hWndList, LVM_ENSUREVISIBLE, i, 0);
                   SendMessageW(This->hWndList, LVM_EDITLABELW, i, 0);
@@ -1524,7 +1531,7 @@ static LRESULT ShellView_OnNotify(IShellViewImpl * This, UINT CtlID, LPNMHDR lpn
 		if (psfhlp == NULL)
 		  break;
 
-		if(!(i = ListView_GetSelectedCount(This->hWndList)))
+		if(!(i = SendMessageW(This->hWndList, LVM_GETSELECTEDCOUNT, 0, 0)))
 		  break;
 
 		/* allocate memory for the pidl array */
@@ -1534,11 +1541,11 @@ static LRESULT ShellView_OnNotify(IShellViewImpl * This, UINT CtlID, LPNMHDR lpn
 		/* retrieve all selected items */
 		i = 0;
 		item_index = -1;
-		while(ListView_GetSelectedCount(This->hWndList) > i)
+		while(SendMessageW(This->hWndList, LVM_GETSELECTEDCOUNT, 0, 0) > i)
 		{
 		  /* get selected item */
-		  item_index = ListView_GetNextItem(This->hWndList,
-			item_index, LVNI_SELECTED);
+		  item_index = SendMessageW(This->hWndList, LVM_GETNEXTITEM, item_index,
+                                            MAKELPARAM (LVNI_SELECTED, 0));
 		  item.iItem = item_index;
 		  item.mask = LVIF_PARAM;
 		  SendMessageA(This->hWndList, LVM_GETITEMA, 0, (LPARAM) &item);
@@ -1693,6 +1700,10 @@ static HRESULT WINAPI IShellView_fnQueryInterface(IShellView2 * iface,REFIID rii
 	{
           *ppvObj = This;
 	}
+	else if(IsEqualIID(riid, &IID_IFolderView))
+	{
+	  *ppvObj = &This->lpvtblFolderView;
+	}
 	else if(IsEqualIID(riid, &IID_IOleCommandTarget))
 	{
           *ppvObj = &This->lpvtblOleCommandTarget;
@@ -1949,54 +1960,26 @@ static HRESULT WINAPI IShellView_fnSaveViewState(IShellView2 * iface)
 static HRESULT WINAPI IShellView_fnSelectItem(
 	IShellView2 * iface,
 	LPCITEMIDLIST pidl,
-	UINT uFlags)
+	UINT flags)
 {
-	IShellViewImpl *This = (IShellViewImpl *)iface;
-	int i;
+    IShellViewImpl *This = (IShellViewImpl *)iface;
+    IFolderView *view;
+    HRESULT hr;
+    int i;
 
-	TRACE("(%p)->(pidl=%p, 0x%08x) stub\n",This, pidl, uFlags);
+    TRACE("(%p)->(pidl=%p, 0x%08x)\n",This, pidl, flags);
 
-	i = LV_FindItemByPidl(This, pidl);
+    i = LV_FindItemByPidl(This, pidl);
+    if (i == -1) return S_OK;
 
-	if (i != -1)
-	{
-	  LVITEMW lvItem;
+    hr = IShellView2_QueryInterface(iface, &IID_IFolderView, (void**)&view);
+    if (hr == S_OK)
+    {
+        hr = IFolderView_SelectItem(view, i, flags);
+        IFolderView_Release(view);
+    }
 
-	  if(uFlags & SVSI_ENSUREVISIBLE)
-	    SendMessageW(This->hWndList, LVM_ENSUREVISIBLE, i, 0);
-
-	  lvItem.mask = LVIF_STATE;
-	  lvItem.stateMask = LVIS_SELECTED | LVIS_FOCUSED;
-	  lvItem.iItem = 0;
-	  lvItem.iSubItem = 0;
-
-          while(SendMessageW(This->hWndList, LVM_GETITEMW, 0, (LPARAM) &lvItem))
-	  {
-	    if (lvItem.iItem == i)
-	    {
-	      if (uFlags & SVSI_SELECT)
-	        lvItem.state |= LVIS_SELECTED;
-	      else
-		lvItem.state &= ~LVIS_SELECTED;
-
-	      if(uFlags & SVSI_FOCUSED)
-	        lvItem.state &= ~LVIS_FOCUSED;
-	    }
-	    else
-	    {
-	      if (uFlags & SVSI_DESELECTOTHERS)
-	        lvItem.state &= ~LVIS_SELECTED;
-	    }
-	    SendMessageW(This->hWndList, LVM_SETITEMW, 0, (LPARAM) &lvItem);
-	    lvItem.iItem++;
-	  }
-
-
-	  if(uFlags & SVSI_EDIT)
-	    SendMessageW(This->hWndList, LVM_EDITLABELW, i, 0);
-
-	}
-	return S_OK;
+    return hr;
 }
 
 static HRESULT WINAPI IShellView_fnGetItemObject(IShellView2 * iface, UINT uItem, REFIID riid, LPVOID *ppvOut)
@@ -2128,10 +2111,26 @@ static HRESULT WINAPI IShellView2_fnHandleRename(IShellView2* iface, LPCITEMIDLI
     return E_NOTIMPL;
 }
 
-static HRESULT WINAPI IShellView2_fnSelectAndPositionItem(IShellView2* iface, LPCITEMIDLIST item, UINT flags, POINT *point)
+static HRESULT WINAPI IShellView2_fnSelectAndPositionItem(
+    IShellView2* iface,
+    LPCITEMIDLIST item,
+    UINT flags,
+    POINT *point)
 {
-    FIXME("(%p)->(item %p, flags %#x, point %p) stub!\n", iface, item, flags, point);
-    return E_NOTIMPL;
+    IShellViewImpl *This = (IShellViewImpl *)iface;
+    IFolderView *view;
+    HRESULT hr;
+
+    TRACE("(%p)->(item %p, flags %#x, point %p)\n", This, item, flags, point);
+
+    hr = IShellView2_QueryInterface(iface, &IID_IFolderView, (void**)&view);
+    if (hr == S_OK)
+    {
+        hr = IFolderView_SelectAndPositionItems(view, 1, &item, point, flags);
+        IFolderView_Release(view);
+    }
+
+    return hr;
 }
 
 static const IShellView2Vtbl svvt =
@@ -2169,7 +2168,7 @@ static HRESULT WINAPI ISVOleCmdTarget_QueryInterface(
 {
 	IShellViewImpl *This = impl_from_IOleCommandTarget(iface);
 
-	return IShellFolder_QueryInterface((IShellFolder*)This, iid, ppvObj);
+	return IShellView2_QueryInterface((IShellView2*)This, iid, ppvObj);
 }
 
 /**********************************************************
@@ -2180,7 +2179,7 @@ static ULONG WINAPI ISVOleCmdTarget_AddRef(
 {
 	IShellViewImpl *This = impl_from_IOleCommandTarget(iface);
 
-	return IShellFolder_AddRef((IShellFolder*)This);
+	return IShellView2_AddRef((IShellView2*)This);
 }
 
 /**********************************************************
@@ -2191,7 +2190,7 @@ static ULONG WINAPI ISVOleCmdTarget_Release(
 {
 	IShellViewImpl *This = impl_from_IOleCommandTarget(iface);
 
-	return IShellFolder_Release((IShellFolder*)This);
+	return IShellView2_Release((IShellView2*)This);
 }
 
 /**********************************************************
@@ -2272,7 +2271,7 @@ static HRESULT WINAPI ISVDropTarget_QueryInterface(
 
 	TRACE("(%p)->(\n\tIID:\t%s,%p)\n",This,debugstr_guid(riid),ppvObj);
 
-	return IShellFolder_QueryInterface((IShellFolder*)This, riid, ppvObj);
+	return IShellView2_QueryInterface((IShellView2*)This, riid, ppvObj);
 }
 
 static ULONG WINAPI ISVDropTarget_AddRef( IDropTarget *iface)
@@ -2281,7 +2280,7 @@ static ULONG WINAPI ISVDropTarget_AddRef( IDropTarget *iface)
 
 	TRACE("(%p)->(count=%u)\n",This,This->ref);
 
-	return IShellFolder_AddRef((IShellFolder*)This);
+	return IShellView2_AddRef((IShellView2*)This);
 }
 
 static ULONG WINAPI ISVDropTarget_Release( IDropTarget *iface)
@@ -2290,7 +2289,7 @@ static ULONG WINAPI ISVDropTarget_Release( IDropTarget *iface)
 
 	TRACE("(%p)->(count=%u)\n",This,This->ref);
 
-	return IShellFolder_Release((IShellFolder*)This);
+	return IShellView2_Release((IShellView2*)This);
 }
 
 /******************************************************************************
@@ -2452,7 +2451,7 @@ static HRESULT WINAPI ISVDropSource_QueryInterface(
 
 	TRACE("(%p)->(\n\tIID:\t%s,%p)\n",This,debugstr_guid(riid),ppvObj);
 
-	return IShellFolder_QueryInterface((IShellFolder*)This, riid, ppvObj);
+	return IShellView2_QueryInterface((IShellView2*)This, riid, ppvObj);
 }
 
 static ULONG WINAPI ISVDropSource_AddRef( IDropSource *iface)
@@ -2461,7 +2460,7 @@ static ULONG WINAPI ISVDropSource_AddRef( IDropSource *iface)
 
 	TRACE("(%p)->(count=%u)\n",This,This->ref);
 
-	return IShellFolder_AddRef((IShellFolder*)This);
+	return IShellView2_AddRef((IShellView2*)This);
 }
 
 static ULONG WINAPI ISVDropSource_Release( IDropSource *iface)
@@ -2470,8 +2469,9 @@ static ULONG WINAPI ISVDropSource_Release( IDropSource *iface)
 
 	TRACE("(%p)->(count=%u)\n",This,This->ref);
 
-	return IShellFolder_Release((IShellFolder*)This);
+	return IShellView2_Release((IShellView2*)This);
 }
+
 static HRESULT WINAPI ISVDropSource_QueryContinueDrag(
 	IDropSource *iface,
 	BOOL fEscapePressed,
@@ -2519,7 +2519,7 @@ static HRESULT WINAPI ISVViewObject_QueryInterface(
 
 	TRACE("(%p)->(\n\tIID:\t%s,%p)\n",This,debugstr_guid(riid),ppvObj);
 
-	return IShellFolder_QueryInterface((IShellFolder*)This, riid, ppvObj);
+	return IShellView2_QueryInterface((IShellView2*)This, riid, ppvObj);
 }
 
 static ULONG WINAPI ISVViewObject_AddRef( IViewObject *iface)
@@ -2528,7 +2528,7 @@ static ULONG WINAPI ISVViewObject_AddRef( IViewObject *iface)
 
 	TRACE("(%p)->(count=%u)\n",This,This->ref);
 
-	return IShellFolder_AddRef((IShellFolder*)This);
+	return IShellView2_AddRef((IShellView2*)This);
 }
 
 static ULONG WINAPI ISVViewObject_Release( IViewObject *iface)
@@ -2537,7 +2537,7 @@ static ULONG WINAPI ISVViewObject_Release( IViewObject *iface)
 
 	TRACE("(%p)->(count=%u)\n",This,This->ref);
 
-	return IShellFolder_Release((IShellFolder*)This);
+	return IShellView2_Release((IShellView2*)This);
 }
 
 static HRESULT WINAPI ISVViewObject_Draw(
@@ -2659,3 +2659,173 @@ static const IViewObjectVtbl vovt =
 	ISVViewObject_SetAdvise,
 	ISVViewObject_GetAdvise
 };
+
+/* IFolderView */
+static HRESULT WINAPI IFView_QueryInterface(
+	IFolderView *iface,
+	REFIID riid,
+	LPVOID *ppvObj)
+{
+	IShellViewImpl *This = impl_from_IFolderView(iface);
+	TRACE("(%p)->(IID:%s,%p)\n", This, debugstr_guid(riid), ppvObj);
+	return IShellView2_QueryInterface((IShellView2*)This, riid, ppvObj);
+}
+
+static ULONG WINAPI IFView_AddRef( IFolderView *iface)
+{
+	IShellViewImpl *This = impl_from_IFolderView(iface);
+	TRACE("(%p)->(count=%u)\n", This, This->ref);
+	return IShellView2_AddRef((IShellView2*)This);
+}
+
+static ULONG WINAPI IFView_Release( IFolderView *iface)
+{
+	IShellViewImpl *This = impl_from_IFolderView(iface);
+	TRACE("(%p)->(count=%u)\n", This, This->ref);
+	return IShellView2_Release((IShellView2*)This);
+}
+
+static HRESULT WINAPI IFView_GetCurrentViewMode(IFolderView *iface, UINT *mode)
+{
+	IShellViewImpl *This = impl_from_IFolderView(iface);
+	FIXME("(%p)->(%p), stub\n", This, mode);
+	return E_NOTIMPL;
+}
+
+static HRESULT WINAPI IFView_SetCurrentViewMode(IFolderView *iface, UINT mode)
+{
+	IShellViewImpl *This = impl_from_IFolderView(iface);
+	FIXME("(%p)->(%u), stub\n", This, mode);
+	return E_NOTIMPL;
+}
+
+static HRESULT WINAPI IFView_GetFolder(IFolderView *iface, REFIID riid, void **ppv)
+{
+	IShellViewImpl *This = impl_from_IFolderView(iface);
+	FIXME("(%p)->(%s, %p), stub\n", This, debugstr_guid(riid), ppv);
+	return E_NOTIMPL;
+}
+
+static HRESULT WINAPI IFView_Item(IFolderView *iface, int index, PITEMID_CHILD *ppidl)
+{
+	IShellViewImpl *This = impl_from_IFolderView(iface);
+	FIXME("(%p)->(%d %p), stub\n", This, index, ppidl);
+	return E_NOTIMPL;
+}
+
+static HRESULT WINAPI IFView_ItemCount(IFolderView *iface, UINT flags, int *items)
+{
+	IShellViewImpl *This = impl_from_IFolderView(iface);
+	FIXME("(%p)->(%u %p), stub\n", This, flags, items);
+	return E_NOTIMPL;
+}
+
+static HRESULT WINAPI IFView_Items(IFolderView *iface, UINT flags, REFIID riid, void **ppv)
+{
+	IShellViewImpl *This = impl_from_IFolderView(iface);
+	FIXME("(%p)->(%u %s %p), stub\n", This, flags, debugstr_guid(riid), ppv);
+	return E_NOTIMPL;
+}
+
+static HRESULT WINAPI IFView_GetSelectionMarkedItem(IFolderView *iface, int *item)
+{
+	IShellViewImpl *This = impl_from_IFolderView(iface);
+	FIXME("(%p)->(%p), stub\n", This, item);
+	return E_NOTIMPL;
+}
+
+static HRESULT WINAPI IFView_GetFocusedItem(IFolderView *iface, int *item)
+{
+	IShellViewImpl *This = impl_from_IFolderView(iface);
+	FIXME("(%p)->(%p), stub\n", This, item);
+	return E_NOTIMPL;
+}
+
+static HRESULT WINAPI IFView_GetItemPosition(IFolderView *iface, PCUITEMID_CHILD pidl, POINT *ppt)
+{
+	IShellViewImpl *This = impl_from_IFolderView(iface);
+	FIXME("(%p)->(%p %p), stub\n", This, pidl, ppt);
+	return E_NOTIMPL;
+}
+
+static HRESULT WINAPI IFView_GetSpacing(IFolderView *iface, POINT *pt)
+{
+	IShellViewImpl *This = impl_from_IFolderView(iface);
+	FIXME("(%p)->(%p), stub\n", This, pt);
+	return E_NOTIMPL;
+}
+
+static HRESULT WINAPI IFView_GetDefaultSpacing(IFolderView *iface, POINT *pt)
+{
+	IShellViewImpl *This = impl_from_IFolderView(iface);
+	FIXME("(%p)->(%p), stub\n", This, pt);
+	return E_NOTIMPL;
+}
+
+static HRESULT WINAPI IFView_GetAutoArrange(IFolderView *iface)
+{
+	IShellViewImpl *This = impl_from_IFolderView(iface);
+	FIXME("(%p), stub\n", This);
+	return E_NOTIMPL;
+}
+
+static HRESULT WINAPI IFView_SelectItem(IFolderView *iface, int item, DWORD flags)
+{
+    IShellViewImpl *This = impl_from_IFolderView(iface);
+    LVITEMW lvItem;
+
+    TRACE("(%p)->(%d, %x)\n", This, item, flags);
+
+    lvItem.state = 0;
+    lvItem.stateMask = LVIS_SELECTED;
+
+    if (flags & SVSI_ENSUREVISIBLE)
+        SendMessageW(This->hWndList, LVM_ENSUREVISIBLE, item, 0);
+
+    /* all items */
+    if (flags & SVSI_DESELECTOTHERS)
+        SendMessageW(This->hWndList, LVM_SETITEMSTATE, -1, (LPARAM)&lvItem);
+
+    /* this item */
+    if (flags & SVSI_SELECT)
+        lvItem.state |= LVIS_SELECTED;
+
+    if (flags & SVSI_FOCUSED)
+        lvItem.stateMask |= LVIS_FOCUSED;
+
+    SendMessageW(This->hWndList, LVM_SETITEMSTATE, item, (LPARAM)&lvItem);
+
+    if (flags & SVSI_EDIT)
+        SendMessageW(This->hWndList, LVM_EDITLABELW, item, 0);
+
+    return S_OK;
+}
+
+static HRESULT WINAPI IFView_SelectAndPositionItems(IFolderView *iface, UINT cidl,
+                                     PCUITEMID_CHILD_ARRAY apidl, POINT *apt, DWORD flags)
+{
+	IShellViewImpl *This = impl_from_IFolderView(iface);
+	FIXME("(%p)->(%u %p %p %x), stub\n", This, cidl, apidl, apt, flags);
+	return E_NOTIMPL;
+}
+
+static const IFolderViewVtbl fviewvt =
+{
+	IFView_QueryInterface,
+	IFView_AddRef,
+	IFView_Release,
+	IFView_GetCurrentViewMode,
+	IFView_SetCurrentViewMode,
+	IFView_GetFolder,
+	IFView_Item,
+	IFView_ItemCount,
+	IFView_Items,
+	IFView_GetSelectionMarkedItem,
+	IFView_GetFocusedItem,
+	IFView_GetItemPosition,
+	IFView_GetSpacing,
+	IFView_GetDefaultSpacing,
+	IFView_GetAutoArrange,
+	IFView_SelectItem,
+	IFView_SelectAndPositionItems
+};
