@@ -177,8 +177,7 @@ static void test_IStream_invalid_operations(IStream * stream, DWORD mode)
 
     /* IStream::Clone */
 
-    ret = IStream_Clone(stream, NULL);
-    ok(ret == E_NOTIMPL, "expected E_NOTIMPL, got 0x%08x\n", ret);
+    /* Passing a NULL pointer for the second IStream::Clone param crashes on Win7 */
 
     clone = NULL;
     ret = IStream_Clone(stream, &clone);
@@ -209,12 +208,16 @@ static void test_SHCreateStreamOnFileA(DWORD mode, DWORD stgm)
     /* invalid arguments */
 
     stream = NULL;
-    /* NT: ERROR_PATH_NOT_FOUND, 9x: ERROR_BAD_PATHNAME */
     ret = (*pSHCreateStreamOnFileA)(NULL, mode | stgm, &stream);
-    ok(ret == HRESULT_FROM_WIN32(ERROR_PATH_NOT_FOUND) ||
-        ret == HRESULT_FROM_WIN32(ERROR_BAD_PATHNAME),
-        "SHCreateStreamOnFileA: expected HRESULT_FROM_WIN32(ERROR_PATH_NOT_FOUND)"
-        "or HRESULT_FROM_WIN32(ERROR_BAD_PATHNAME), got 0x%08x\n", ret);
+    if (ret == E_INVALIDARG) /* Win98 SE */ {
+        win_skip("Not supported\n");
+        return;
+    }
+
+    ok(ret == HRESULT_FROM_WIN32(ERROR_PATH_NOT_FOUND) /* NT */ ||
+       ret == HRESULT_FROM_WIN32(ERROR_BAD_PATHNAME) /* 9x */,
+       "SHCreateStreamOnFileA: expected HRESULT_FROM_WIN32(ERROR_PATH_NOT_FOUND) "
+       "or HRESULT_FROM_WIN32(ERROR_BAD_PATHNAME), got 0x%08x\n", ret);
     ok(stream == NULL, "SHCreateStreamOnFileA: expected a NULL IStream object, got %p\n", stream);
 
 #if 0 /* This test crashes on WinXP SP2 */
@@ -306,17 +309,23 @@ static void test_SHCreateStreamOnFileW(DWORD mode, DWORD stgm)
 
     /* invalid arguments */
 
-    stream = NULL;
-    ret = (*pSHCreateStreamOnFileW)(NULL, mode | stgm, &stream);
-    ok(ret == HRESULT_FROM_WIN32(ERROR_PATH_NOT_FOUND) || /* XP */
-       ret == E_INVALIDARG /* Vista */,
-      "SHCreateStreamOnFileW: expected HRESULT_FROM_WIN32(ERROR_PATH_NOT_FOUND) or E_INVALIDARG, got 0x%08x\n", ret);
-    ok(stream == NULL, "SHCreateStreamOnFileW: expected a NULL IStream object, got %p\n", stream);
+    if (0)
+    {
+        /* Crashes on NT4 */
+        stream = NULL;
+        ret = (*pSHCreateStreamOnFileW)(NULL, mode | stgm, &stream);
+        ok(ret == HRESULT_FROM_WIN32(ERROR_PATH_NOT_FOUND) || /* XP */
+           ret == E_INVALIDARG /* Vista */,
+          "SHCreateStreamOnFileW: expected HRESULT_FROM_WIN32(ERROR_PATH_NOT_FOUND) or E_INVALIDARG, got 0x%08x\n", ret);
+        ok(stream == NULL, "SHCreateStreamOnFileW: expected a NULL IStream object, got %p\n", stream);
+    }
 
-#if 0 /* This test crashes on WinXP SP2 */
-    ret = (*pSHCreateStreamOnFileW)(test_file, mode | stgm, NULL);
-    ok(ret == E_INVALIDARG, "SHCreateStreamOnFileW: expected E_INVALIDARG, got 0x%08x\n", ret);
-#endif
+    if (0)
+    {
+        /* This test crashes on WinXP SP2 */
+            ret = (*pSHCreateStreamOnFileW)(test_file, mode | stgm, NULL);
+            ok(ret == E_INVALIDARG, "SHCreateStreamOnFileW: expected E_INVALIDARG, got 0x%08x\n", ret);
+    }
 
     stream = NULL;
     ret = (*pSHCreateStreamOnFileW)(test_file, mode | STGM_CONVERT | stgm, &stream);
@@ -337,6 +346,11 @@ static void test_SHCreateStreamOnFileW(DWORD mode, DWORD stgm)
 
     stream = NULL;
     ret = (*pSHCreateStreamOnFileW)(test_file, mode | STGM_FAILIFTHERE | stgm, &stream);
+    if (ret == E_INVALIDARG) /* Win98 SE */ {
+        win_skip("Not supported\n");
+        return;
+    }
+
     ok(ret == HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND), "SHCreateStreamOnFileW: expected HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND), got 0x%08x\n", ret);
     ok(stream == NULL, "SHCreateStreamOnFileW: expected a NULL IStream object, got %p\n", stream);
 
@@ -405,12 +419,16 @@ static void test_SHCreateStreamOnFileEx(DWORD mode, DWORD stgm)
 
     /* invalid arguments */
 
-    stream = NULL;
-    ret = (*pSHCreateStreamOnFileEx)(NULL, mode, 0, FALSE, NULL, &stream);
-    ok(ret == HRESULT_FROM_WIN32(ERROR_PATH_NOT_FOUND) || /* XP */
-       ret == E_INVALIDARG /* Vista */,
-      "SHCreateStreamOnFileEx: expected HRESULT_FROM_WIN32(ERROR_PATH_NOT_FOUND) or E_INVALIDARG, got 0x%08x\n", ret);
-    ok(stream == NULL, "SHCreateStreamOnFileEx: expected a NULL IStream object, got %p\n", stream);
+    if (0)
+    {
+        /* Crashes on NT4 */
+        stream = NULL;
+        ret = (*pSHCreateStreamOnFileEx)(NULL, mode, 0, FALSE, NULL, &stream);
+        ok(ret == HRESULT_FROM_WIN32(ERROR_PATH_NOT_FOUND) || /* XP */
+           ret == E_INVALIDARG /* Vista */,
+          "SHCreateStreamOnFileEx: expected HRESULT_FROM_WIN32(ERROR_PATH_NOT_FOUND) or E_INVALIDARG, got 0x%08x\n", ret);
+        ok(stream == NULL, "SHCreateStreamOnFileEx: expected a NULL IStream object, got %p\n", stream);
+    }
 
     stream = NULL;
     ret = (*pSHCreateStreamOnFileEx)(test_file, mode, 0, FALSE, template, &stream);
@@ -421,10 +439,12 @@ static void test_SHCreateStreamOnFileEx(DWORD mode, DWORD stgm)
 
     ok(stream == NULL, "SHCreateStreamOnFileEx: expected a NULL IStream object, got %p\n", stream);
 
-#if 0 /* This test crashes on WinXP SP2 */
-    ret = (*pSHCreateStreamOnFileEx)(test_file, mode, 0, FALSE, NULL, NULL);
-    ok(ret == E_INVALIDARG, "SHCreateStreamOnFileEx: expected E_INVALIDARG, got 0x%08x\n", ret);
-#endif
+    if (0)
+    {
+        /* This test crashes on WinXP SP2 */
+        ret = (*pSHCreateStreamOnFileEx)(test_file, mode, 0, FALSE, NULL, NULL);
+        ok(ret == E_INVALIDARG, "SHCreateStreamOnFileEx: expected E_INVALIDARG, got 0x%08x\n", ret);
+    }
 
     /* file does not exist */
 
