@@ -1061,9 +1061,9 @@ static DWORD buffer_sanitize_flags(DWORD flags)
 
 static GLbitfield buffer_gl_map_flags(DWORD d3d_flags)
 {
-    GLbitfield ret = GL_MAP_FLUSH_EXPLICIT_BIT;
+    GLbitfield ret = 0;
 
-    if (!(d3d_flags & WINED3DLOCK_READONLY)) ret |= GL_MAP_WRITE_BIT;
+    if (!(d3d_flags & WINED3DLOCK_READONLY)) ret = GL_MAP_WRITE_BIT | GL_MAP_FLUSH_EXPLICIT_BIT;
 
     if (d3d_flags & (WINED3DLOCK_DISCARD | WINED3DLOCK_NOOVERWRITE))
     {
@@ -1086,7 +1086,10 @@ static HRESULT STDMETHODCALLTYPE buffer_Map(IWineD3DBuffer *iface, UINT offset,
     TRACE("iface %p, offset %u, size %u, data %p, flags %#x\n", iface, offset, size, data, flags);
 
     flags = buffer_sanitize_flags(flags);
-    if (!buffer_add_dirty_area(This, offset, size)) return E_OUTOFMEMORY;
+    if (!(flags & WINED3DLOCK_READONLY))
+    {
+        if (!buffer_add_dirty_area(This, offset, size)) return E_OUTOFMEMORY;
+    }
 
     count = InterlockedIncrement(&This->lock_count);
 
