@@ -467,10 +467,11 @@ static HRESULT shader_get_registers_used(IWineD3DBaseShader *iface, const struct
     {
         struct wined3d_shader_instruction ins;
         const char *comment;
+        UINT comment_size;
         UINT param_size;
 
         /* Skip comments. */
-        fe->shader_read_comment(&ptr, &comment);
+        fe->shader_read_comment(&ptr, &comment, &comment_size);
         if (comment) continue;
 
         /* Fetch opcode. */
@@ -1115,10 +1116,11 @@ void shader_generate_main(IWineD3DBaseShader *iface, struct wined3d_shader_buffe
     while (!fe->shader_is_end(fe_data, &ptr))
     {
         const char *comment;
+        UINT comment_size;
         UINT param_size;
 
         /* Skip comment tokens. */
-        fe->shader_read_comment(&ptr, &comment);
+        fe->shader_read_comment(&ptr, &comment, &comment_size);
         if (comment) continue;
 
         /* Read opcode. */
@@ -1222,13 +1224,34 @@ static void shader_trace_init(const struct wined3d_shader_frontend *fe, void *fe
     {
         struct wined3d_shader_instruction ins;
         const char *comment;
+        UINT comment_size;
         UINT param_size;
 
         /* comment */
-        fe->shader_read_comment(&ptr, &comment);
+        fe->shader_read_comment(&ptr, &comment, &comment_size);
         if (comment)
         {
-            TRACE("//%s\n", comment);
+            if (comment_size > 4 && *(const DWORD *)comment == WINEMAKEFOURCC('T', 'E', 'X', 'T'))
+            {
+                const char *end = comment + comment_size;
+                const char *ptr = comment + 4;
+                const char *line = ptr;
+
+                TRACE("// TEXT\n");
+                while (ptr != end)
+                {
+                    if (*ptr == '\n')
+                    {
+                        UINT len = ptr - line;
+                        if (len && *(ptr - 1) == '\r') --len;
+                        TRACE("// %s\n", debugstr_an(line, len));
+                        line = ++ptr;
+                    }
+                    else ++ptr;
+                }
+                if (line != ptr) TRACE("// %s\n", debugstr_an(line, ptr - line));
+            }
+            else TRACE("// %s\n", debugstr_an(comment, comment_size));
             continue;
         }
 
@@ -1404,7 +1427,7 @@ static BOOL shader_none_color_fixup_supported(struct color_fixup_desc fixup)
     }
 
     /* Faked to make some apps happy. */
-    if (!is_yuv_fixup(fixup))
+    if (!is_complex_fixup(fixup))
     {
         TRACE("[OK]\n");
         return TRUE;
