@@ -142,6 +142,31 @@ static void remove_startup_notification(Display *display, Window window)
 }
 
 
+struct has_popup_result
+{
+    HWND hwnd;
+    BOOL found;
+};
+
+static BOOL CALLBACK has_popup( HWND hwnd, LPARAM lparam )
+{
+    struct has_popup_result *result = (struct has_popup_result *)lparam;
+
+    if (hwnd == result->hwnd) return FALSE;  /* popups are always above owner */
+    result->found = (GetWindow( hwnd, GW_OWNER ) == result->hwnd);
+    return !result->found;
+}
+
+static BOOL has_owned_popups( HWND hwnd )
+{
+    struct has_popup_result result;
+
+    result.hwnd = hwnd;
+    result.found = FALSE;
+    EnumWindows( has_popup, (LPARAM)&result );
+    return result.found;
+}
+
 /***********************************************************************
  *		is_window_managed
  *
@@ -181,6 +206,8 @@ static BOOL is_window_managed( HWND hwnd, UINT swp_flags, const RECT *window_rec
     /* application windows are managed */
     ex_style = GetWindowLongW( hwnd, GWL_EXSTYLE );
     if (ex_style & WS_EX_APPWINDOW) return TRUE;
+    /* windows that own popups are managed */
+    if (has_owned_popups( hwnd )) return TRUE;
     /* default: not managed */
     return FALSE;
 }
@@ -917,7 +944,7 @@ static void set_initial_wm_hints( Display *display, struct x11drv_win_data *data
 {
     long i;
     Atom protocols[3];
-    Atom dndVersion = 4;
+    Atom dndVersion = WINE_XDND_VERSION;
     XClassHint *class_hints;
     char *process_name = get_process_name();
 
@@ -965,6 +992,33 @@ static void set_initial_wm_hints( Display *display, struct x11drv_win_data *data
 }
 
 
+/***********************************************************************
+ *              get_owner_whole_window
+ *
+ * Retrieve an owner's window, creating it if necessary.
+ */
+static Window get_owner_whole_window( HWND owner )
+{
+    struct x11drv_win_data *data;
+
+    if (!owner) return 0;
+
+    if (!(data = X11DRV_get_win_data( owner )))
+    {
+        if (GetWindowThreadProcessId( owner, NULL ) != GetCurrentThreadId() ||
+            !(data = X11DRV_create_win_data( owner )))
+            return (Window)GetPropA( owner, whole_window_prop );
+    }
+    else if (!data->managed)  /* make it managed */
+    {
+        SetWindowPos( owner, 0, 0, 0, 0, 0,
+                      SWP_NOACTIVATE | SWP_NOZORDER | SWP_NOSIZE | SWP_NOMOVE |
+                      SWP_NOREDRAW | SWP_DEFERERASE | SWP_NOSENDCHANGING | SWP_STATECHANGED );
+    }
+    return data->whole_window;
+}
+
+
 /***********************************************************************
  *              set_wm_hints
  *
@@ -972,7 +1026,8 @@ static void set_initial_wm_hints( Display *display, struct x11drv_win_data *data
  */
 static void set_wm_hints( Display *display, struct x11drv_win_data *data )
 {
-    Window group_leader;
+    Window group_leader = data->whole_window;
+    Window owner_win = 0;
     Atom window_type;
     MwmHints mwm_hints;
     DWORD style, ex_style;
@@ -990,21 +1045,13 @@ static void set_wm_hints( Display *display, struct x11drv_win_data *data )
         style = GetWindowLongW( data->hwnd, GWL_STYLE );
         ex_style = GetWindowLongW( data->hwnd, GWL_EXSTYLE );
         owner = get_window_owner( data->hwnd );
+        if ((owner_win = get_owner_whole_window( owner ))) group_leader = owner_win;
     }
 
-    /* transient for hint */
-    if (owner)
-    {
-        Window owner_win = X11DRV_get_whole_window( owner );
-        wine_tsx11_lock();
-        XSetTransientForHint( display, data->whole_window, owner_win );
-        wine_tsx11_unlock();
-        group_leader = owner_win;
-    }
-    else group_leader = data->whole_window;
-
     wine_tsx11_lock();
 
+    if (owner_win) XSetTransientForHint( display, data->whole_window, owner_win );
+
     /* size hints */
     set_size_hints( display, data, style );
 
@@ -1277,8 +1324,8 @@ void X11DRV_X_to_window_rect( struct x11drv_win_data *data, RECT *rect )
  * Synchronize the X window position with the Windows one
  */
 static void sync_window_position( Display *display, struct x11drv_win_data *data,
-                                  UINT swp_flags, const RECT *old_client_rect,
-                                  const RECT *old_whole_rect )
+                                  UINT swp_flags, const RECT *old_window_rect,
+                                  const RECT *old_whole_rect, const RECT *old_client_rect )
 {
     DWORD style = GetWindowLongW( data->hwnd, GWL_STYLE );
     XWindowChanges changes;
@@ -1329,10 +1376,13 @@ static void sync_window_position( Display *display, struct x11drv_win_data *data
 #ifdef HAVE_LIBXSHAPE
     if (data->shaped)
     {
-        int x_offset = old_whole_rect->left - data->whole_rect.left;
-        int y_offset = old_whole_rect->top - data->whole_rect.top;
-        if (x_offset || y_offset)
-            XShapeOffsetShape( display, data->whole_window, ShapeBounding, x_offset, y_offset );
+        int old_x_offset = old_window_rect->left - old_whole_rect->left;
+        int old_y_offset = old_window_rect->top - old_whole_rect->top;
+        int new_x_offset = data->window_rect.left - data->whole_rect.left;
+        int new_y_offset = data->window_rect.top - data->whole_rect.top;
+        if (old_x_offset != new_x_offset || old_y_offset != new_y_offset)
+            XShapeOffsetShape( display, data->whole_window, ShapeBounding,
+                               new_x_offset - old_x_offset, new_y_offset - old_y_offset );
     }
 #endif
     wine_tsx11_unlock();
@@ -1953,6 +2003,7 @@ void CDECL X11DRV_ReleaseDC( HWND hwnd, HDC hdc )
     escape.drawable_rect = virtual_screen_rect;
     SetRect( &escape.dc_rect, 0, 0, virtual_screen_rect.right - virtual_screen_rect.left,
              virtual_screen_rect.bottom - virtual_screen_rect.top );
+    OffsetRect( &escape.dc_rect, -escape.drawable_rect.left, -escape.drawable_rect.top );
     escape.fbconfig_id = 0;
     escape.gl_drawable = 0;
     escape.pixmap = 0;
@@ -2097,7 +2148,7 @@ void CDECL X11DRV_WindowPosChanged( HWND hwnd, HWND insert_after, UINT swp_flags
     Display *display;
     struct x11drv_win_data *data = X11DRV_get_win_data( hwnd );
     DWORD new_style = GetWindowLongW( hwnd, GWL_STYLE );
-    RECT old_whole_rect, old_client_rect;
+    RECT old_window_rect, old_whole_rect, old_client_rect;
     int event_type;
 
     if (!data) return;
@@ -2105,6 +2156,7 @@ void CDECL X11DRV_WindowPosChanged( HWND hwnd, HWND insert_after, UINT swp_flags
     thread_data = x11drv_thread_data();
     display = thread_data->display;
 
+    old_window_rect = data->window_rect;
     old_whole_rect  = data->whole_rect;
     old_client_rect = data->client_rect;
     data->window_rect = *rectWindow;
@@ -2163,7 +2215,8 @@ void CDECL X11DRV_WindowPosChanged( HWND hwnd, HWND insert_after, UINT swp_flags
     /* don't change position if we are about to minimize or maximize a managed window */
     if (!event_type &&
         !(data->managed && (swp_flags & SWP_STATECHANGED) && (new_style & (WS_MINIMIZE|WS_MAXIMIZE))))
-        sync_window_position( display, data, swp_flags, &old_client_rect, &old_whole_rect );
+        sync_window_position( display, data, swp_flags,
+                              &old_window_rect, &old_whole_rect, &old_client_rect );
 
     if ((new_style & WS_VISIBLE) &&
         ((new_style & WS_MINIMIZE) || is_window_rect_mapped( rectWindow )))
