@@ -41,8 +41,8 @@
 extern "C" {
 #endif
 
-unsigned long __cdecl __threadid(void);
-unsigned long __cdecl __threadhandle(void);
+__msvcrt_ulong __cdecl __threadid(void);
+__msvcrt_ulong __cdecl __threadhandle(void);
 #define _threadid    (__threadid())
 
 #ifdef __cplusplus
