@@ -5,6 +5,7 @@
  *  Copyright 2002 Sylvain Petreolle <spetreolle@yahoo.fr>
  *  Copyright 2002 Andriy Palamarchuk
  *  Copyright 2007 Rolf Kalbermatter
+ *  Copyright 2010 Vitaly Perov
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -337,24 +338,6 @@ static inline ENCODING detect_encoding_of_buffer(const void* buffer, int size)
     }
 }
 
-/* Similar to SetWindowTextA, but uses a CP_UTF8 encoded input, not CP_ACP.
- * lpTextInUtf8 should be NUL-terminated and not include the BOM.
- *
- * Returns FALSE on failure, TRUE on success, like SetWindowTextA/W.
- */
-static BOOL SetWindowTextUtf8(HWND hwnd, LPCSTR lpTextInUtf8)
-{
-    BOOL ret;
-    int lenW = MultiByteToWideChar(CP_UTF8, 0, lpTextInUtf8, -1, NULL, 0);
-    LPWSTR textW = HeapAlloc(GetProcessHeap(), 0, lenW * sizeof(WCHAR));
-    if (!textW)
-        return FALSE;
-    MultiByteToWideChar(CP_UTF8, 0, lpTextInUtf8, -1, textW, lenW);
-    ret = SetWindowTextW(hwnd, textW);
-    HeapFree(GetProcessHeap(), 0, textW);
-    return ret;
-}
-
 void DoOpenFile(LPCWSTR szFileName, ENCODING enc)
 {
     static const WCHAR dotlog[] = { '.','L','O','G',0 };
@@ -362,7 +345,9 @@ void DoOpenFile(LPCWSTR szFileName, ENCODING enc)
     LPSTR pTemp;
     DWORD size;
     DWORD dwNumRead;
-    BOOL succeeded;
+    int lenW;
+    WCHAR* textW;
+    int i;
     WCHAR log[5];
 
     /* Close any files and prompt to save changes */
@@ -405,8 +390,6 @@ void DoOpenFile(LPCWSTR szFileName, ENCODING enc)
     CloseHandle(hFile);
 
     size = dwNumRead;
-    pTemp[size] = 0;    /* make sure it's  (char)'\0'-terminated */
-    pTemp[size+1] = 0;  /* make sure it's (WCHAR)'\0'-terminated */
 
     if (enc == ENCODING_AUTO)
         enc = detect_encoding_of_buffer(pTemp, size);
@@ -421,9 +404,6 @@ void DoOpenFile(LPCWSTR szFileName, ENCODING enc)
             enc = ENCODING_UTF16BE;
     }
 
-    /* SetWindowTextUtf8 and SetWindowTextA try to allocate memory, so we
-     * check if they succeed.
-     */
     switch (enc)
     {
     case ENCODING_UTF16BE:
@@ -434,33 +414,41 @@ void DoOpenFile(LPCWSTR szFileName, ENCODING enc)
         /* fall through */
 
     case ENCODING_UTF16LE:
-        if (size >= 2 && (BYTE)pTemp[0] == 0xff && (BYTE)pTemp[1] == 0xfe)
-            succeeded = SetWindowTextW(Globals.hEdit, (LPWSTR)pTemp + 1);
-        else
-            succeeded = SetWindowTextW(Globals.hEdit, (LPWSTR)pTemp);
-        break;
-
-    case ENCODING_UTF8:
-        if (size >= 3 && (BYTE)pTemp[0] == 0xef && (BYTE)pTemp[1] == 0xbb &&
-                                                   (BYTE)pTemp[2] == 0xbf)
-            succeeded = SetWindowTextUtf8(Globals.hEdit, pTemp+3);
-        else
-            succeeded = SetWindowTextUtf8(Globals.hEdit, pTemp);
+        textW = (LPWSTR)pTemp;
+        lenW  = size/sizeof(WCHAR);
         break;
 
     default:
-        succeeded = SetWindowTextA(Globals.hEdit, pTemp);
-        break;
+        {
+            int cp = (enc==ENCODING_UTF8) ? CP_UTF8 : CP_ACP;
+            lenW = MultiByteToWideChar(cp, 0, pTemp, size, NULL, 0);
+            textW = HeapAlloc(GetProcessHeap(), 0, (lenW+1) * sizeof(WCHAR));
+            if (!textW)
+            {
+                ShowLastError();
+                HeapFree(GetProcessHeap(), 0, pTemp);
+                return;
+            }
+            MultiByteToWideChar(cp, 0, pTemp, size, textW, lenW);
+            HeapFree(GetProcessHeap(), 0, pTemp);
+            break;
+        }
     }
 
-    if (!succeeded)
-    {
-        ShowLastError();
-        HeapFree(GetProcessHeap(), 0, pTemp);
-        return;
-    }
+    /* Replace '\0's with spaces. Other than creating a custom control that
+     * can deal with '\0' characters, it's the best that can be done.
+     */
+    for (i = 0; i < lenW; i++)
+        if (textW[i] == '\0')
+            textW[i] = ' ';
+    textW[lenW] = '\0';
 
-    HeapFree(GetProcessHeap(), 0, pTemp);
+    if (lenW >= 1 && textW[0] == 0xfeff)
+        SetWindowTextW(Globals.hEdit, textW+1);
+    else
+        SetWindowTextW(Globals.hEdit, textW);
+
+    HeapFree(GetProcessHeap(), 0, textW);
 
     SendMessageW(Globals.hEdit, EM_SETMODIFY, FALSE, 0);
     SendMessageW(Globals.hEdit, EM_EMPTYUNDOBUFFER, 0, 0);
@@ -522,6 +510,26 @@ static ENCODING detect_encoding_of_file(LPCWSTR szFileName)
     }
 }
 
+static LPWSTR dialog_print_to_file(HWND hMainWnd)
+{
+    OPENFILENAMEW ofn;
+    static WCHAR file[MAX_PATH] = {'o','u','t','p','u','t','.','p','r','n',0};
+    static const WCHAR defExt[] = {'p','r','n',0};
+
+    ZeroMemory(&ofn, sizeof(ofn));
+
+    ofn.lStructSize = sizeof(ofn);
+    ofn.Flags = OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
+    ofn.hwndOwner = hMainWnd;
+    ofn.lpstrFile = file;
+    ofn.nMaxFile = MAX_PATH;
+    ofn.lpstrDefExt = defExt;
+
+    if(GetSaveFileNameW(&ofn))
+        return file;
+    else
+        return FALSE;
+}
 static UINT_PTR CALLBACK OfnHookProc(HWND hdlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
     static HWND hEncCombo;
@@ -848,6 +856,13 @@ VOID DIALOG_FilePrint(VOID)
     di.lpszDatatype = NULL;
     di.fwType = 0; 
 
+    if(printer.Flags & PD_PRINTTOFILE)
+    {
+        di.lpszOutput = dialog_print_to_file(printer.hwndOwner);
+        if(!di.lpszOutput)
+            return;
+    }
+
     /* Get the file text */
     size = GetWindowTextLengthW(Globals.hEdit) + 1;
     pTemp = HeapAlloc(GetProcessHeap(), 0, size * sizeof(WCHAR));
