@@ -23,11 +23,12 @@ enum be_xpoint_type {be_xpoint_break, be_xpoint_watch_exec, be_xpoint_watch_read
                      be_xpoint_watch_write};
 struct backend_cpu
 {
+    DWORD               machine;
     /* ------------------------------------------------------------------------------
      * address manipulation
      * ------------------------------------------------------------------------------ */
     /* Linearizes an address. Only CPUs with segmented address model need this.
-     * Otherwise, implementation is straigthforward (be_cpu_linearize will do)
+     * Otherwise, implementation is straightforward (be_cpu_linearize will do)
      */
     void*               (*linearize)(HANDLE hThread, const ADDRESS64*);
     /* Fills in an ADDRESS64 structure from a segment & an offset. CPUs without
@@ -76,9 +77,9 @@ struct backend_cpu
      * it's INT3 (0xCC)
      */
     unsigned            (*is_break_insn)(const void*);
-    /* Check whether instruciton at 'addr' is a function call */
+    /* Check whether instruction at 'addr' is a function call */
     unsigned            (*is_function_call)(const void* insn, ADDRESS64* callee);
-    /* Ask for dissasembling one instruction. If display is true, assembly code
+    /* Ask for disassembling one instruction. If display is true, assembly code
      * will be printed. In all cases, 'addr' is advanced at next instruction
      */
     void                (*disasm_one_insn)(ADDRESS64* addr, int display);
