@@ -66,7 +66,7 @@ textStatus (va_list ap)
 
     fputs (str, stderr);
     fputc ('\n', stderr);
-    free (str);
+    heap_free (str);
     return 0;
 }
 
@@ -78,7 +78,7 @@ guiStatus (va_list ap)
 
     if (len > 128) str[129] = 0;
     SetDlgItemText (dialog, IDC_SB, str);
-    free (str);
+    heap_free (str);
     return 0;
 }
 
@@ -108,7 +108,7 @@ guiProgress (va_list ap)
     else progressScale = 1;
     pb = GetDlgItem (dialog, IDC_PB0 + progressGroup * 2);
     SendMessage (pb, PBM_SETRANGE, 0, MAKELPARAM (0, max));
-    SendMessage (pb, PBM_SETSTEP, (WPARAM)1, 0);
+    SendMessage (pb, PBM_SETSTEP, 1, 0);
     return 0;
 }
 
@@ -121,7 +121,7 @@ textStep (va_list ap)
     progressCurr++;
     fputs (str, stderr);
     fprintf (stderr, " (%d of %d)\n", progressCurr, progressMax);
-    free (str);
+    heap_free (str);
     return 0;
 }
 
@@ -134,8 +134,8 @@ guiStep (va_list ap)
     progressCurr++;
     SetDlgItemText (dialog, pgID, str);
     SendDlgItemMessage (dialog, pgID+1, PBM_SETPOS,
-                        (WPARAM)(progressScale * progressCurr), 0);
-    free (str);
+                        progressScale * progressCurr, 0);
+    heap_free (str);
     return 0;
 }
 
@@ -149,7 +149,7 @@ textDelta (va_list ap)
     progressCurr += inc;
     fputs (str, stderr);
     fprintf (stderr, " (%d of %d)\n", progressCurr, progressMax);
-    free (str);
+    heap_free (str);
     return 0;
 }
 
@@ -163,8 +163,8 @@ guiDelta (va_list ap)
     progressCurr += inc;
     SetDlgItemText (dialog, pgID, str);
     SendDlgItemMessage (dialog, pgID+1, PBM_SETPOS,
-                        (WPARAM)(progressScale * progressCurr), 0);
-    free (str);
+                        progressScale * progressCurr, 0);
+    heap_free (str);
     return 0;
 }
 
@@ -194,7 +194,7 @@ textDir (va_list ap)
     fputs ("Temporary directory: ", stderr);
     fputs (str, stderr);
     fputc ('\n', stderr);
-    free (str);
+    heap_free (str);
     return 0;
 }
 
@@ -204,7 +204,7 @@ guiDir (va_list ap)
     char *str = vstrmake (NULL, ap);
 
     SetDlgItemText (dialog, IDC_DIR, str);
-    free (str);
+    heap_free (str);
     return 0;
 }
 
@@ -217,7 +217,7 @@ textOut (va_list ap)
     fputs ("Log file: ", stderr);
     fputs (str, stderr);
     fputc ('\n', stderr);
-    free (str);
+    heap_free (str);
     return 0;
 }
 
@@ -227,7 +227,7 @@ guiOut (va_list ap)
     char *str = vstrmake (NULL, ap);
 
     SetDlgItemText (dialog, IDC_OUT, str);
-    free (str);
+    heap_free (str);
     return 0;
 }
 
@@ -246,7 +246,7 @@ guiWarning (va_list ap)
     char *str = vstrmake (NULL, ap);
 
     MessageBox (dialog, str, "Warning", MB_ICONWARNING | MB_OK);
-    free (str);
+    heap_free (str);
     return 0;
 }
 
@@ -265,7 +265,7 @@ guiError (va_list ap)
     char *str = vstrmake (NULL, ap);
 
     MessageBox (dialog, str, "Error", MB_ICONERROR | MB_OK);
-    free (str);
+    heap_free (str);
     return 0;
 }
 
@@ -294,7 +294,7 @@ textAsk (va_list ap)
 
     fprintf (stderr, "Question of type %d: %s\n"
              "Returning default: %d\n", uType, str, ret);
-    free (str);
+    heap_free (str);
     return ret;
 }
 
@@ -306,7 +306,7 @@ guiAsk (va_list ap)
     int ret = MessageBox (dialog, str, "Question",
                           MB_ICONQUESTION | uType);
 
-    free (str);
+    heap_free (str);
     return ret;
 }
 
@@ -341,7 +341,7 @@ AskTagProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
                report (R_WARNING, "You must enter a tag to continue");
                return FALSE;
             }
-            tag = xmalloc (len+1);
+            tag = heap_alloc (len+1);
             GetDlgItemTextA (hwnd, IDC_TAG, tag, len+1);
             EndDialog (hwnd, IDOK);
             return TRUE;
@@ -361,6 +361,39 @@ guiAskTag (void)
                       dialog, AskTagProc);
 }
 
+static INT_PTR CALLBACK
+AskEmailProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
+{
+    int len;
+
+    switch (msg)
+    {
+    case WM_COMMAND:
+        switch (LOWORD (wParam)) {
+        case IDOK:
+	    len = GetWindowTextLengthA (GetDlgItem (hwnd, IDC_EMAIL));
+	    if(!len) {
+               report (R_WARNING, "You must enter an email address to continue");
+               return FALSE;
+            }
+            email = heap_alloc (len+1);
+            GetDlgItemTextA (hwnd, IDC_EMAIL, email, len+1);
+            EndDialog (hwnd, IDOK);
+            return TRUE;
+        case IDABORT:
+            EndDialog (hwnd, IDABORT);
+            return TRUE;
+        }
+    }
+    return FALSE;
+}
+
+int
+guiAskEmail (void)
+{
+    return DialogBox (GetModuleHandle (NULL), MAKEINTRESOURCE (IDD_EMAIL), dialog, AskEmailProc);
+}
+
 /* Quiet functions */
 static int
 qNoOp (va_list ap)
@@ -440,7 +473,7 @@ DlgThreadProc (LPVOID param)
                      NULL, DlgProc);
     switch (ret) {
     case 0:
-        report (R_WARNING, "Invalid parent handle");
+        report (R_FATAL, "Cannot display dialog");
         break;
     case 1:
         report (R_WARNING, "DialogBox failed: %d",
