@@ -63,6 +63,7 @@ static HWND hMainWnd;
 static HWND hEditorWnd;
 static HWND hFindWnd;
 static HMENU hPopupMenu;
+static HMENU hColorPopupMenu;
 
 static UINT ID_FINDMSGSTRING;
 
@@ -1836,17 +1837,18 @@ static LRESULT OnCreate( HWND hWnd )
 
     hFormatBarWnd = CreateToolbarEx(hReBarWnd,
          CCS_NOPARENTALIGN | CCS_NOMOVEY | WS_VISIBLE | TBSTYLE_TOOLTIPS | TBSTYLE_BUTTON,
-         IDC_FORMATBAR, 7, hInstance, IDB_FORMATBAR, NULL, 0, 16, 16, 16, 16, sizeof(TBBUTTON));
+         IDC_FORMATBAR, 8, hInstance, IDB_FORMATBAR, NULL, 0, 16, 16, 16, 16, sizeof(TBBUTTON));
 
     AddButton(hFormatBarWnd, 0, ID_FORMAT_BOLD);
     AddButton(hFormatBarWnd, 1, ID_FORMAT_ITALIC);
     AddButton(hFormatBarWnd, 2, ID_FORMAT_UNDERLINE);
+    AddButton(hFormatBarWnd, 3, ID_FORMAT_COLOR);
     AddSeparator(hFormatBarWnd);
-    AddButton(hFormatBarWnd, 3, ID_ALIGN_LEFT);
-    AddButton(hFormatBarWnd, 4, ID_ALIGN_CENTER);
-    AddButton(hFormatBarWnd, 5, ID_ALIGN_RIGHT);
+    AddButton(hFormatBarWnd, 4, ID_ALIGN_LEFT);
+    AddButton(hFormatBarWnd, 5, ID_ALIGN_CENTER);
+    AddButton(hFormatBarWnd, 6, ID_ALIGN_RIGHT);
     AddSeparator(hFormatBarWnd);
-    AddButton(hFormatBarWnd, 6, ID_BULLET);
+    AddButton(hFormatBarWnd, 7, ID_BULLET);
 
     SendMessageW(hFormatBarWnd, TB_AUTOSIZE, 0, 0);
 
@@ -2001,6 +2003,15 @@ static LRESULT OnNotify( HWND hWnd, LPARAM lParam)
     return 0;
 }
 
+/* Copied from dlls/comdlg32/fontdlg.c */
+static const COLORREF textcolors[]=
+{
+    0x00000000L,0x00000080L,0x00008000L,0x00008080L,
+    0x00800000L,0x00800080L,0x00808000L,0x00808080L,
+    0x00c0c0c0L,0x000000ffL,0x0000ff00L,0x0000ffffL,
+    0x00ff0000L,0x00ff00ffL,0x00ffff00L,0x00FFFFFFL
+};
+
 static LRESULT OnCommand( HWND hWnd, WPARAM wParam, LPARAM lParam)
 {
     HWND hwndEditor = GetDlgItem(hWnd, IDC_EDITOR);
@@ -2098,7 +2109,7 @@ static LRESULT OnCommand( HWND hWnd, WPARAM wParam, LPARAM lParam)
         break;
 
     case ID_PRINT_QUICK:
-        print_quick(wszFileName);
+        print_quick(hMainWnd, wszFileName);
         target_device(hMainWnd, wordWrap[reg_formatindex(fileFormat)]);
         break;
 
@@ -2106,7 +2117,7 @@ static LRESULT OnCommand( HWND hWnd, WPARAM wParam, LPARAM lParam)
         {
             int index = reg_formatindex(fileFormat);
             DWORD tmp = barState[index];
-            barState[index] = 0;
+            barState[index] = 1 << BANDID_STATUSBAR;
             set_bar_states();
             barState[index] = tmp;
             ShowWindow(hEditorWnd, FALSE);
@@ -2152,6 +2163,46 @@ static LRESULT OnCommand( HWND hWnd, WPARAM wParam, LPARAM lParam)
         break;
         }
 
+    case ID_FORMAT_COLOR:
+    {
+        HWND hReBarWnd = GetDlgItem(hWnd, IDC_REBAR);
+        HWND hFormatBarWnd = GetDlgItem(hReBarWnd, IDC_FORMATBAR);
+        HMENU hPop;
+        RECT itemrc;
+        POINT pt;
+        int mid;
+        int itemidx = SendMessage(hFormatBarWnd, TB_COMMANDTOINDEX, ID_FORMAT_COLOR, 0);
+
+        SendMessage(hFormatBarWnd, TB_GETITEMRECT, itemidx, (LPARAM)&itemrc);
+        pt.x = itemrc.left;
+        pt.y = itemrc.bottom;
+        ClientToScreen(hFormatBarWnd, &pt);
+        hPop = GetSubMenu(hColorPopupMenu, 0);
+        mid = TrackPopupMenu(hPop, TPM_LEFTALIGN | TPM_TOPALIGN | TPM_LEFTBUTTON |
+                                   TPM_RETURNCMD | TPM_NONOTIFY,
+                             pt.x, pt.y, 0, hWnd, 0);
+        if (mid >= ID_COLOR_FIRST && mid <= ID_COLOR_AUTOMATIC)
+        {
+            CHARFORMAT2W fmt;
+
+            ZeroMemory(&fmt, sizeof(fmt));
+            fmt.cbSize = sizeof(fmt);
+            SendMessageW(hwndEditor, EM_GETCHARFORMAT, SCF_SELECTION, (LPARAM)&fmt);
+
+            fmt.dwMask = CFM_COLOR;
+
+            if (mid < ID_COLOR_AUTOMATIC) {
+                fmt.crTextColor = textcolors[mid - ID_COLOR_FIRST];
+                fmt.dwEffects &= ~CFE_AUTOCOLOR;
+            } else {
+                fmt.dwEffects |= CFE_AUTOCOLOR;
+            }
+
+            SendMessageW(hwndEditor, EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&fmt);
+        }
+        break;
+    }
+
     case ID_EDIT_CUT:
         PostMessageW(hwndEditor, WM_CUT, 0, 0);
         break;
@@ -2618,6 +2669,7 @@ int CALLBACK WinMain(HINSTANCE hInstance, HINSTANCE hOldInstance, LPSTR szCmdPar
     set_bar_states();
     set_fileformat(SF_RTF);
     hPopupMenu = LoadMenuW(hInstance, MAKEINTRESOURCEW(IDM_POPUP));
+    hColorPopupMenu = LoadMenuW(hInstance, MAKEINTRESOURCEW(IDM_COLOR_POPUP));
     get_default_printer_opts();
     target_device(hMainWnd, wordWrap[reg_formatindex(fileFormat)]);
 
