@@ -420,16 +420,10 @@ void generate_debug_event( struct thread *thread, int code, const void *arg )
 /* attach a process to a debugger thread and suspend it */
 static int debugger_attach( struct process *process, struct thread *debugger )
 {
-    struct thread *thread;
-
     if (process->debugger) goto error;  /* already being debugged */
     if (!is_process_init_done( process )) goto error;  /* still starting up */
     if (list_empty( &process->thread_list )) goto error;  /* no thread running in the process */
 
-    /* make sure we don't create a debugging loop */
-    for (thread = debugger; thread; thread = thread->process->debugger)
-        if (thread->process == process) goto error;
-
     /* don't let a debugger debug its console... won't work */
     if (debugger->process->console && console_get_renderer(debugger->process->console)->process == process)
         goto error;
@@ -699,7 +693,6 @@ DECL_HANDLER(output_debug_string)
     debug_event_t data;
 
     data.output_string.string  = req->string;
-    data.output_string.unicode = req->unicode;
     data.output_string.length  = req->length;
     generate_debug_event( current, OUTPUT_DEBUG_STRING_EVENT, &data );
 }
