@@ -328,7 +328,6 @@ struct thread *create_process( int fd, struct thread *parent_thread, int inherit
     process->startup_state   = STARTUP_IN_PROGRESS;
     process->startup_info    = NULL;
     process->idle_event      = NULL;
-    process->queue           = NULL;
     process->peb             = 0;
     process->ldt_copy        = 0;
     process->winstation      = 0;
@@ -423,7 +422,6 @@ static void process_destroy( struct object *obj )
     if (process->msg_fd) release_object( process->msg_fd );
     list_remove( &process->entry );
     if (process->idle_event) release_object( process->idle_event );
-    if (process->queue) release_object( process->queue );
     if (process->id) free_ptid( process->id );
     if (process->token) release_object( process->token );
 }
@@ -564,23 +562,17 @@ static void process_unload_dll( struct process *process, mod_handle_t base )
 /* terminate a process with the given exit code */
 static void terminate_process( struct process *process, struct thread *skip, int exit_code )
 {
-    struct list *ptr;
-
-    if (skip && skip->process == process)  /* move it to the end of the list */
-    {
-        assert( skip->state != TERMINATED );
-        list_remove( &skip->proc_entry );
-        list_add_tail( &process->thread_list, &skip->proc_entry );
-    }
+    struct thread *thread;
 
     grab_object( process );  /* make sure it doesn't get freed when threads die */
-    while ((ptr = list_head( &process->thread_list )))
+restart:
+    LIST_FOR_EACH_ENTRY( thread, &process->thread_list, struct thread, proc_entry )
     {
-        struct thread *thread = LIST_ENTRY( ptr, struct thread, proc_entry );
-
         if (exit_code) thread->exit_code = exit_code;
-        if (thread == skip) break;
+        if (thread == skip) continue;
+        if (thread->state == TERMINATED) continue;
         kill_thread( thread, 1 );
+        goto restart;
     }
     release_object( process );
 }
@@ -632,6 +624,13 @@ static void process_killed( struct process *process )
     handles = process->handles;
     process->handles = NULL;
     if (handles) release_object( handles );
+    process->winstation = 0;
+    process->desktop = 0;
+    if (process->idle_event)
+    {
+        release_object( process->idle_event );
+        process->idle_event = NULL;
+    }
 
     /* close the console attached to this process, if any */
     free_console( process );
@@ -645,6 +644,7 @@ static void process_killed( struct process *process )
         free( dll );
     }
     destroy_process_classes( process );
+    free_process_user_handles( process );
     remove_process_locks( process );
     set_process_startup_state( process, STARTUP_ABORTED );
     finish_process_tracing( process );
@@ -1095,6 +1095,18 @@ DECL_HANDLER(get_process_info)
     }
 }
 
+static void set_process_affinity( struct process *process, affinity_t affinity )
+{
+    struct thread *thread;
+
+    process->affinity = affinity;
+
+    LIST_FOR_EACH_ENTRY( thread, &process->thread_list, struct thread, proc_entry )
+    {
+        set_thread_affinity( thread, affinity );
+    }
+}
+
 /* set information about a process */
 DECL_HANDLER(set_process_info)
 {
@@ -1103,7 +1115,7 @@ DECL_HANDLER(set_process_info)
     if ((process = get_process_from_handle( req->handle, PROCESS_SET_INFORMATION )))
     {
         if (req->mask & SET_PROCESS_INFO_PRIORITY) process->priority = req->priority;
-        if (req->mask & SET_PROCESS_INFO_AFFINITY) process->affinity = req->affinity;
+        if (req->mask & SET_PROCESS_INFO_AFFINITY) set_process_affinity( process, req->affinity );
         release_object( process );
     }
 }
