@@ -636,7 +636,7 @@ static void reply_message( struct msg_queue *queue, lparam_t result,
 static int match_window( user_handle_t win, user_handle_t msg_win )
 {
     if (!win) return 1;
-    if (win == (user_handle_t)-1) return !msg_win;
+    if (win == -1 || win == 1) return !msg_win;
     if (msg_win == win) return 1;
     return is_child_window( win, msg_win );
 }
@@ -1819,6 +1819,7 @@ DECL_HANDLER(get_message)
         return;
     }
 
+    if (get_win == -1 && current->process->idle_event) set_event( current->process->idle_event );
     queue->wake_mask = req->wake_mask;
     queue->changed_mask = req->changed_mask;
     set_error( STATUS_PENDING );  /* FIXME */
