@@ -118,7 +118,8 @@ extern int thread_add_inflight_fd( struct thread *thread, int client, int server
 extern int thread_get_inflight_fd( struct thread *thread, int client );
 extern struct thread_snapshot *thread_snap( int *count );
 extern struct token *thread_get_impersonation_token( struct thread *thread );
-extern void set_thread_affinity( struct thread *thread, affinity_t affinity );
+extern int is_wow64_thread( struct thread *thread );
+extern int set_thread_affinity( struct thread *thread, affinity_t affinity );
 
 /* ptrace functions */
 
