@@ -72,6 +72,7 @@ typedef struct list statement_list_t;
 enum attr_type
 {
     ATTR_AGGREGATABLE,
+    ATTR_ANNOTATION,
     ATTR_APPOBJECT,
     ATTR_ASYNC,
     ATTR_AUTO_HANDLE,
@@ -120,6 +121,7 @@ enum attr_type
     ATTR_OLEAUTOMATION,
     ATTR_OPTIONAL,
     ATTR_OUT,
+    ATTR_PARAMLCID,
     ATTR_POINTERDEFAULT,
     ATTR_POINTERTYPE,
     ATTR_PROPGET,
@@ -184,6 +186,7 @@ enum expr_type
     EXPR_POS,
     EXPR_STRLIT,
     EXPR_WSTRLIT,
+    EXPR_CHARCONST,
 };
 
 enum type_kind
@@ -228,6 +231,7 @@ enum type_basic_type
     TYPE_BASIC_INT32,
     TYPE_BASIC_INT64,
     TYPE_BASIC_INT,
+    TYPE_BASIC_INT3264,
     TYPE_BASIC_CHAR,
     TYPE_BASIC_HYPER,
     TYPE_BASIC_BYTE,
@@ -340,6 +344,12 @@ struct pointer_details
   unsigned char def_fc;
 };
 
+struct bitfield_details
+{
+  struct _type_t *field;
+  const expr_t *bits;
+};
+
 enum type_type
 {
     TYPE_VOID,
@@ -355,6 +365,7 @@ enum type_type
     TYPE_INTERFACE,
     TYPE_POINTER,
     TYPE_ARRAY,
+    TYPE_BITFIELD,
 };
 
 struct _type_t {
@@ -372,6 +383,7 @@ struct _type_t {
     struct coclass_details coclass;
     struct basic_details basic;
     struct pointer_details pointer;
+    struct bitfield_details bitfield;
   } details;
   type_t *orig;                   /* dup'd types */
   unsigned int typestring_offset;
@@ -405,6 +417,7 @@ struct _declarator_t {
   type_t *type;
   type_t *func_type;
   array_dims_t *array;
+  expr_t *bits;
 
   /* parser-internal */
   struct list entry;
@@ -487,6 +500,14 @@ struct _statement_t {
     } u;
 };
 
+typedef enum {
+    SYS_WIN16,
+    SYS_WIN32,
+    SYS_MAC,
+    SYS_WIN64
+} syskind_t;
+
+extern syskind_t typelib_kind;
 extern user_type_list_t user_type_list;
 void check_for_additional_prototype_types(const var_list_t *list);
 
